/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PageEventListener.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.page;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.EventException;


/**
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public abstract class PageEventListener extends DefaultBaseEventListener {

    private static final Logger logger = Logger.getLogger(PageEventListener.class.getName());

    public final void handleControlEvent(ControlEventContext context)
            throws EventException, ServletException, IOException {
        HttpServletRequest req = context.getRequest();
        HttpSession session = req.getSession(false);

        Page page = null;
        //track state of pages; new OR cached and differs from Page defined for
        //non-auxilary current event would make this true. Default to false,
        //meaning cached and same as that defined for the current event.
        boolean pageNewOrDiffers = false;
        if (session != null) {
            page = (Page)session.getAttribute(PageFactory.PAGE_KEY);
        }
        if (page == null) {
            page = (Page)context.getState(PageFactory.PAGE_KEY);
        }

        BaseEvent event = context.getEvent();
        if (logger.isDebugEnabled()) logger.debug("Processing event " + event.getClass().getName());
        if (page == null) {
            pageNewOrDiffers = true;
            if (logger.isDebugEnabled()) logger.debug("No cached PAGE found, need to create fresh PAGE for event");
            page = PageFactory.createPage(event);
            if (page != null) {
                if (logger.isDebugEnabled()) logger.debug("Page creation successful!");
                page.createDefaultModels(context);
            } else {
                //a null pointer exception would occur below if we didn't throw this exception...
                String errMsg = "Failed to create fresh PAGE for event";
                logger.fatal(errMsg);
                throw new PageException(errMsg);
            }
        } else {
            if (logger.isDebugEnabled()) logger.debug("Cached Page found, checking if it differs from the one specified for event");
            if (!page.getClass().equals(PageFactory.getPageClass(event))) {
                pageNewOrDiffers = true;
                if (logger.isDebugEnabled()) logger.debug("Page differs, may need to create fresh Page to hand over for event");
                Page newPage = PageFactory.createPage(event);
                if (newPage != null) {
                    if (logger.isInfoEnabled()) logger.info("Handing over");
                    newPage.handOver(context, page);
                    page = newPage;
                } else {
                    pageNewOrDiffers = false;
                    //this condition happens when the given event is auxilary,
                    //meaning it is not mapped to a page. Only functional
                    //events are mapped to pages.  See javadoc comments in
                    //PageFactory.createPage(BaseEvent) for details.
                }
            } else {
                if (logger.isInfoEnabled()) logger.info("Cached Page class matches that associated with requested event " + event.getClass().getName() + ", using cached Page");
            }
        }

        if (page instanceof SessionPage) {
            if (logger.isDebugEnabled()) logger.debug("Page " + page.getClass().getName() + " implements SessionPage");
            if (session == null) {
                //Need to do this or a session might never be created to store session pages.
                //This is because whenever we check for a session page, we attempt to get the
                //session but tell theh request object not to create one if it doesn't exist
                //already.  So, the only way a session would exist, other than doing this, is
                //if another part of the running application happens to create a session. That's
                //a bit too hit-or-miss. Need to be sure to support SessionPage functionality.
                if (logger.isDebugEnabled()) logger.debug("Creating session for SessionPage storage");
                session = req.getSession(true);
            }
            if (pageNewOrDiffers) {
                //avoid re-setting the same object in the session constantly.  No need to
                //set it if it is already there.
                if (logger.isDebugEnabled()) logger.debug("Storing Page in the HttpSession");
                session.setAttribute(PageFactory.PAGE_KEY, page);
            }
        } else {
            if (pageNewOrDiffers && session != null) {
                //only need to bother removing the attribute from an existing session if the page
                //is not the same as one stored in cache for the current event.  If it is the same
                //page as before and we get into this conditional block, then we know the last page
                //wasn't a SessionPage and, therefore it is impossible for there to be a page stored
                //in the session. It would have been previously removed the last time the page was
                //different from that stored in cache for the current event.
                if (logger.isDebugEnabled()) logger.debug("Ensuring any existing SessionPage is Removed from the HttpSession");
                session.removeAttribute(PageFactory.PAGE_KEY);
            }
            context.putState(PageFactory.PAGE_KEY, page);
        }

        handleControlEvent(context, page);
    }

    public abstract void handleControlEvent(ControlEventContext context, Page page)
            throws EventException, ServletException, IOException;

}
