/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PageFactory.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.page;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.BaseEvent;


/**
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public class PageFactory {

    public static final String PAGE_KEY = PageFactory.class.getName() + ".PAGE";
    public static final String SYNCHEE = PageFactory.class.getName() + ".SYNCHEE";

    private static final Logger logger = Logger.getLogger(PageFactory.class.getName());
    private static Map s_pageClasses = new HashMap();

    /**
     * Maps events to page objects. The event class is expected to be
     * assignment-compatible with a
     * {@link org.barracudamvc.core.event.BaseEvent} object and the page
     * class assignment-compatible with a {@link Page} object. If these
     * conditions are not met, the mapping will not take place.
     *
     * @param event a class assignment-compatible with a BaseEvent object
     * @param page a class assignment-compatible with a Page object
     */
    public static void addPageEventMapping(Class event, Class page) {
        if (BaseEvent.class.isAssignableFrom(event) && Page.class.isAssignableFrom(page)) {
            s_pageClasses.put(event, page);
            if (logger.isInfoEnabled()) logger.info("Registered page " + page.getName() + " for event " + event.getName());
        } else {
            logger.error("Failed to register page with event...");
            if (!BaseEvent.class.isAssignableFrom(event)) logger.error("Class " + event.getName() + " is not an assignment-compatible with " + BaseEvent.class.getName());
            if (!Page.class.isAssignableFrom(page)) logger.error("Class " + page.getName() + " is not an assignment-compatible with " + Page.class.getName());
        }
    }

    /**
     * Creates pages for events which have been mapped to pages via
     * {@link #addPageEventMapping(Class, Class)}. This method returns a
     * page object only for functional events which represent
     * an actual screen presentation.  <code>null</code> will be returned for
     * auxilary events which are responsible for dealing with data that is
     * common to all or a subset of screens, such as navigation bars, footers,
     * headers, teaserlists and so on. They never get their own page - but they
     * expect the pages they deal with to implement a certain interface and,
     * more so, maybe even the underlying DOM (this concept is mostly
     * associated with <a href="http://xmlc.enhydra.org/">XMLC</a>) class.
     *
     * @param event the current event
     * @return a Page object for functional events and null for auxilary events
     */
    public static Page createPage(BaseEvent event) {
        if (s_pageClasses.containsKey(event.getClass())) {
            Class pageClass = getPageClass(event);
            if (pageClass != null) {
                if (logger.isInfoEnabled()) logger.info("Creating page for event " + event.getClass().getName());
                if (logger.isDebugEnabled()) {
                    logger.debug("All Registered pages...");
                    logger.debug(s_pageClasses);
                }
                try {
                    return (Page)pageClass.newInstance();
                } catch (InstantiationException ie) {
                    logger.error(ie);
                } catch (IllegalAccessException iae) {
                    logger.error(iae);
                }
            } else {
                //this should never happen!
                logger.error("Couldn't find page for event: " + event.getClass().getName());
            }
        } else {
            if (logger.isDebugEnabled()) logger.debug("Event may be auxilary, no page registered for event, no new page created");
        }

        //TODO: Error handling
        return null;
    }

    /**
     * provides access to the {@link Page} class associated with the given
     * event which has been mapped to the page via
     * {@link #addPageEventMapping(Class, Class)}.
     *
     * @return a class which is assignment-compatible with a Page object
     */
    public static Class getPageClass(BaseEvent event) {
        return (Class)s_pageClasses.get(event.getClass());
    }

}
