/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ErrorFormMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.repopulation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BText;
import org.barracudamvc.core.forms.DefaultFormMap;
import org.barracudamvc.core.forms.FormElement;
import org.barracudamvc.core.forms.FormMap;
import org.barracudamvc.core.forms.ValidationException;

/**
 * @author Diez B. Roggisch <diez.roggisch@artnology.com>
 * This class eases the population of errors resulting from form-validation.
 * This is done by catching raised exceptions and if their sources are
 * FormElements, a mapping is made which consists of the static ERROR_PREFIX
 * and the FormElements name to a BComponent.<p>
 * If the Exception has set a text, the mapped component is a BText-component
 * with that text. Otherwise it's just a BComponent.<p>
 * When using this class, you should do two things:<p>
 * <ol>
 * <li>Subclass it and implement the TemplateModel-interface</li>
 * <li>Add ERROR_PREFIX + Element-names directives to your HTML-code, e.g <b>Dir::Get_Data.ModelName.e_Password</b></li>
 * </ol>
 * When processing the BTemplate, these directives will return null for
 * elements without errors, resulting in discarding the nodes.
 * Or they return the associated components so the nodes stay, and text will be substituted.
 */
public class ErrorFormMap extends DefaultFormMap
{
    private Map errorMap = new HashMap();
    public static String ERROR_PREFIX = "e_";
    protected static final Logger logger = Logger.getLogger(ErrorFormMap.class.getName());


    public FormMap validate(boolean deferExceptions) throws ValidationException {
                errorMap.clear();
                try {
                        super.validate(deferExceptions);
                }
                catch(ValidationException ex) {
                        if(logger.isDebugEnabled()) {
                                logger.debug("Catched ValidationException");
                        }
                        List e_list;
                        if(deferExceptions) {
                                e_list = ex.getExceptionList();
                        }
                        else {
                                e_list = new Vector();
                                e_list.add(ex);
                        }

                        for(int i = 0; i < e_list.size(); i++) {
                                if(logger.isDebugEnabled()) {
                                        logger.debug("Visiting Exception " + i);
                                }
                                ValidationException vex = (ValidationException)e_list.get(i);
                                if(vex.getSource() instanceof FormElement) {
                                        FormElement fel = (FormElement)vex.getSource();
                                        BComponent comp;

                                        logger.debug("Element name: " + fel.getName());
                                        if(vex.getMessage() != null && !vex.getMessage().equals("")) {
                                                if(logger.isDebugEnabled()) {
                                                        logger.debug("Adding BText with " + vex.getMessage());
                                                }
                                                comp = new BText();
                                                ((BText)comp).setText(vex.getMessage());
                                        }
                                        else {
                                                if(logger.isDebugEnabled()) {
                                                        logger.debug("Adding BComponent");
                                                }
                                                comp = new BComponent();
                                        }
                                        errorMap.put(ERROR_PREFIX + fel.getKey(), comp);
                                }
                        }
                        throw ex;
                }
                return this;
    }


    /** If you want errors from whole-form-validation to be populated, simply use this as a
     * convenient method to add mappings.
     */
    public void addErrorComponent(String key, BComponent comp) {
                errorMap.put(key, comp);
    }

    /** In your subclasses TemplateModel-interface call this.
     */
    public Object getErrorComponent(String key)
    {
                if(errorMap.containsKey(key)){
                        return errorMap.get(key);
                }
                return null;
    }
}
