/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DataObjectHandler.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.data;

import javax.servlet.http.HttpServletRequest;
import org.apache.log4j.Logger;
import org.barracudamvc.contrib.sam.models.ErrorModel;
import org.barracudamvc.contrib.sam.xmlform.XmlFormMap;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.InterruptDispatchException;
import org.barracudamvc.core.event.ViewEvent;
import org.barracudamvc.core.forms.ValidationException;

/** Base class for other DataHandlers. DataObjectHandler instatiates the XML form
 * map specified by it's Config. If the form is already visited, validation is
 * performed.
 */
public abstract class DataObjectHandler extends DefaultBaseEventListener implements GetConfig {

	protected static Logger logger = Logger.getLogger(DataObjectHandler.class.getName());

	public abstract Config getConfig();
	protected Config config;

	/** <ul>
	 * <li> get a DataObject based on the OID parameter given in the request
	 * <li> instatiate a XMLFormMap based on a XML file given in the Config
	 * <li> if the form is already visited, try to perform validation
	 * <li> if validation fails, create a ErrorModel
	 * <li> continue processing by calling the Config's ViewEvent
	 */
	public void handleControlEvent(ControlEventContext context)
		throws EventException, javax.servlet.ServletException, java.io.IOException {
		BaseEvent event = context.getEvent();
		HttpServletRequest request = context.getRequest();
		logger.info("request is " + request.getClass().getName() + " content-type: " + request.getContentType() + " param:  " + request.getHeader("Content-type"));
		
		config = getConfig();

		if (logger.isDebugEnabled())
			logger.debug("handling controlEvent " + event.getClass().getName());
		DataObject dataObj = null;
		try {
			dataObj = config.getDataObject(request.getParameter(config.getOidName()));
		} catch (DataObjectException ex) {
			logger.error(ex.getMessage());
		}

		controlEventHookBeforeXmlFormMap(context, dataObj);

		XmlFormMap xfm = new XmlFormMap(context, config.getXmlFormName(), config.getFormName(), dataObj);
		context.putState(config.getMapName(), xfm);
		if (logger.isDebugEnabled())
			logger.debug("putting session attribute " + config.getMapName() + " value: " + xfm);

		if (config.getFormName().equals(request.getParameter("visited"))) {
			try {
				logger.debug("Mapping form");
				xfm.map(request).validate(true);
			} catch (ValidationException ex) {
				ErrorModel.create(context, ex);
				logger.error(ex.getMessage(), ex);
				throw new InterruptDispatchException(config.getViewEvent());
			}
		}
		controlEventHookAfterXmlFormMap(context, dataObj);

		ViewEvent renderEvent = config.getViewEvent();
		if (logger.isDebugEnabled())
			logger.debug("putting " + renderEvent.getClass().getName() + " to event queue");
		event.setHandled(true);
		context.getQueue().addEvent(renderEvent);
	}

    /** hook funtion to be overridden in subclasses, empty implementation. It is called before the XMLFormMap is instantiated */
    protected void controlEventHookBeforeXmlFormMap(ControlEventContext context, DataObject dataObj) {
    }

    /** hook funtion to be overridden in subclasses, empty implementation. It is called after the XMLFormMap is instantiated and validated */
    protected void controlEventHookAfterXmlFormMap(ControlEventContext context, DataObject dataObj) {
    }
}
