/*
 * Copyright (C) 2003 Stefan Armbruster [stefan@armbruster-it.de]
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: XmlFormViewHandler.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.xmlform;

import java.io.IOException;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.enhydra.apache.xml.serialize.HTMLSerializer;
import org.enhydra.apache.xml.serialize.OutputFormat;
import org.barracudamvc.contrib.sam.xmlform.dtd.Element;
import org.barracudamvc.contrib.sam.xmlform.dtd.Form;
import org.barracudamvc.contrib.sam.xmlform.dtd.Ref_id;
import org.barracudamvc.contrib.sam.xmlform.dtd.Text;
import org.barracudamvc.contrib.sam.xmlform.dtd.Textarea;
import org.barracudamvc.contrib.sam.xmlform.dtd.Upload;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.ViewContext;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLAnchorElement;
import org.w3c.dom.html.HTMLElement;
import org.w3c.dom.html.HTMLInputElement;
import org.w3c.dom.html.HTMLSelectElement;
import org.w3c.dom.html.HTMLTextAreaElement;

/**
 * extends ModifyDomViewHandler in order to automatically creation of a XML Form based view.
 */
public abstract class XmlFormViewHandler extends ModifyDomViewHandler {

    public static final String UPLOAD_LINK_POSTFIX = "_link";

    protected static Logger logger = Logger.getLogger(XmlFormViewHandler.class.getName());

    /** overwrite in subclasses, must return a XmlFormMap defining the fields of the form */
    protected abstract XmlFormMap getXmlForm(ViewContext vc);

    /**
     * needed because getTemplateModels does not have a context
     */
    public Object getTemplateModels(ViewContext vc) {
        List models = new Vector();
        XmlFormMap xfm = getXmlForm(vc);
        models.add(xfm);

        List cModels = xfm.getChildModels();
        for (Iterator it = cModels.iterator(); it.hasNext();) {
            models.add((TemplateModel) it.next());
        }

        return models;
    }

    /**
     * helper function for inserting a input element into the dom
     */
    private void insertIntoDOM(Node parent, Node node, boolean useInsertBefore, Node insert) {
        if (useInsertBefore)
            parent.insertBefore(node, insert);
        else
            parent.appendChild(node);
    }

    /**
     * change the given Document, copy the templates of the various form types as defined in the XmlFormMap
     */
    protected void modifyDOM(Document page, ViewContext vc) {
        XmlFormMap xmlForm = getXmlForm(vc);

        org.w3c.dom.Element textNode = null;
        HTMLElement textNameNode = null;
        HTMLElement textLabelNode = null;
        HTMLInputElement textInputNode = null;
        org.w3c.dom.Element uploadNode = null;
        HTMLElement uploadNameNode = null;
        HTMLElement uploadLabelNode = null;
        HTMLInputElement uploadInputNode = null;
        HTMLAnchorElement uploadLinkNode = null;
        org.w3c.dom.Element selectNode = null;
        HTMLElement selectNameNode = null;
        HTMLElement selectLabelNode = null;
        HTMLSelectElement selectInputNode = null;
        org.w3c.dom.Element textareaNode = null;
        HTMLElement textareaNameNode = null;
        HTMLElement textareaLabelNode = null;
        HTMLTextAreaElement textareaInputNode = null;
        org.w3c.dom.Element radioNode = null;
        HTMLElement radioNameNode = null;
        HTMLElement radioLabelNode = null;
        HTMLElement radioValueLabelNode = null;
        HTMLInputElement radioInputNode = null;
        org.w3c.dom.Element checkboxNode = null;
        HTMLElement checkboxNameNode = null;
        HTMLElement checkboxLabelNode = null;
        HTMLElement checkboxValueLabelNode = null;
        HTMLInputElement checkboxInputNode = null;
        HTMLElement checkboxGroupNode = null;

        // get references to the template nodes for varios form field types
        Form form = xmlForm.getForm();
        Iterator iter = form.getRef_idList().iterator();
        while (iter.hasNext()) {
            Ref_id r = (Ref_id) iter.next();
            String type = r.getType();
            if (type.equals("text")) {
                textNode = page.getElementById(r.getId());
                if (r.getName_id() != null) textNameNode = (HTMLElement) page.getElementById(r.getName_id());
                textLabelNode = (HTMLElement) page.getElementById(r.getLabel_id());
                textInputNode = (HTMLInputElement) page.getElementById(r.getInput_id());
            } else if (type.equals("upload")) {
                uploadNode = page.getElementById(r.getId());
                if (r.getName_id() != null) uploadNameNode = (HTMLElement) page.getElementById(r.getName_id());
                uploadLabelNode = (HTMLElement) page.getElementById(r.getLabel_id());
                uploadInputNode = (HTMLInputElement) page.getElementById(r.getInput_id());
                uploadLinkNode = (HTMLAnchorElement) page.getElementById(r.getLink_id());
            } else if (type.equals("select")) {
                selectNode = page.getElementById(r.getId());
                if (r.getName_id() != null) selectNameNode = (HTMLElement) page.getElementById(r.getName_id());
                selectLabelNode = (HTMLElement) page.getElementById(r.getLabel_id());
                selectInputNode = (HTMLSelectElement) page.getElementById(r.getInput_id());
            } else if (type.equals("textarea")) {
                textareaNode = page.getElementById(r.getId());
                if (r.getName_id() != null) textareaNameNode = (HTMLElement) page.getElementById(r.getName_id());
                textareaLabelNode = (HTMLElement) page.getElementById(r.getLabel_id());
                textareaInputNode = (HTMLTextAreaElement) page.getElementById(r.getInput_id());
            } else if (type.equals("radio")) {
                radioNode = page.getElementById(r.getId());
                if (r.getName_id() != null) radioNameNode = (HTMLElement) page.getElementById(r.getName_id());
                radioLabelNode = (HTMLElement) page.getElementById(r.getLabel_id());
                radioInputNode = (HTMLInputElement) page.getElementById(r.getInput_id());
                radioValueLabelNode = (HTMLElement) page.getElementById(r.getValuelabel_id());
                if (radioValueLabelNode == null) radioValueLabelNode = (HTMLElement) radioInputNode.getNextSibling();
            } else if (type.equals("checkbox")) {
                checkboxNode = page.getElementById(r.getId());
                if (r.getName_id() != null) checkboxNameNode = (HTMLElement) page.getElementById(r.getName_id());
                checkboxLabelNode = (HTMLElement) page.getElementById(r.getLabel_id());
                checkboxInputNode = (HTMLInputElement) page.getElementById(r.getInput_id());
                checkboxValueLabelNode = (HTMLElement) page.getElementById(r.getValuelabel_id());
                checkboxGroupNode = (HTMLElement) page.getElementById(r.getGroup_id());
                if (checkboxValueLabelNode == null)
                        checkboxValueLabelNode = (HTMLElement) checkboxInputNode.getNextSibling();
                if (checkboxInputNode == null) logger.warn("no element with id " + r.getValuelabel_id() + " found");
            }

        }

        // determine insertion mode: insertBefore or appendChild
        boolean isInsertBefore = (form.getInsertBefore() != null);
        Node insert = null;
        Node parent = null;
        if (isInsertBefore) {
            insert = page.getElementById(form.getInsertBefore().getId());
            parent = page.getElementById(form.getInsertBefore().getParent());
        } else
            parent = page.getElementById(form.getAppendChild().getId());

        // all id attributes need to be removed, since id is unique in a document
        if (textNode != null) textNode.removeAttribute("id");
        if (textNameNode != null) textNameNode.removeAttribute("id");
        if (textLabelNode != null) textLabelNode.removeAttribute("id");
        if (textInputNode != null) textInputNode.removeAttribute("id");
        if (uploadNode != null) uploadNode.removeAttribute("id");
        if (uploadNameNode != null) uploadNameNode.removeAttribute("id");
        if (uploadLabelNode != null) uploadLabelNode.removeAttribute("id");
        if (uploadInputNode != null) uploadInputNode.removeAttribute("id");
        if (uploadLinkNode != null) uploadLinkNode.removeAttribute("id");
        if (selectNode != null) selectNode.removeAttribute("id");
        if (selectNameNode != null) selectNameNode.removeAttribute("id");
        if (selectLabelNode != null) selectLabelNode.removeAttribute("id");
        if (selectInputNode != null) selectInputNode.removeAttribute("id");
        if (textareaNode != null) textareaNode.removeAttribute("id");
        if (textareaNameNode != null) textareaNameNode.removeAttribute("id");
        if (textareaLabelNode != null) textareaLabelNode.removeAttribute("id");
        if (textareaInputNode != null) textareaInputNode.removeAttribute("id");
        if (radioNode != null) radioNode.removeAttribute("id");
        if (radioNameNode != null) radioNameNode.removeAttribute("id");
        if (radioLabelNode != null) radioLabelNode.removeAttribute("id");
        if (radioValueLabelNode != null) radioValueLabelNode.removeAttribute("id");
        if (radioInputNode != null) radioInputNode.removeAttribute("id");
        if (checkboxNode != null) checkboxNode.removeAttribute("id");
        if (checkboxNameNode != null) checkboxNameNode.removeAttribute("id");
        if (checkboxLabelNode != null) checkboxLabelNode.removeAttribute("id");
        if (checkboxValueLabelNode != null) checkboxValueLabelNode.removeAttribute("id");
        if (checkboxInputNode != null) checkboxInputNode.removeAttribute("id");
        if (checkboxGroupNode != null) checkboxGroupNode.removeAttribute("id");

        // all preparation is done, so do the dirty work:
        // for each form element, find the matching form type template and copy it
        iter = form.getElementList().iterator();
        while (iter.hasNext()) {
            Element ele = (Element) iter.next();
            //if (!ele.getName().startsWith(EntryFormMap.PREFIX)) {
            if (ele.getText() != null) {
                if (logger.isDebugEnabled()) logger.debug("creating a text element");
                Text t = ele.getText();
                if (textNameNode != null) textNameNode.getFirstChild().setNodeValue(ele.getName());
                textLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                if (t.getMaxlength() != null)
                    textInputNode.setMaxLength(Integer.parseInt(t.getMaxlength()));
                else
                    textInputNode.removeAttribute("maxlength");
                textInputNode.setSize(t.getSize());
                textInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                insertIntoDOM(parent, textNode.cloneNode(true), isInsertBefore, insert);
            } else if (ele.getUpload() != null) {
                if (logger.isDebugEnabled()) logger.debug("creating a upload element");
                Upload u = ele.getUpload();
                if (uploadNameNode != null) uploadNameNode.getFirstChild().setNodeValue(ele.getName());
                if (uploadLabelNode != null) uploadLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                if (u.getMaxlength() != null)
                    uploadInputNode.setMaxLength(Integer.parseInt(u.getMaxlength()));
                else
                    uploadInputNode.removeAttribute("maxlength");
                uploadInputNode.setSize(u.getSize());
                uploadInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                // set link
                if (uploadLinkNode != null) {
                    logger.debug("creating a upload link element");
                    /*
                     * String href = u.getLink_event() + "?field=" + ele.getName() + "&cnt_field=" +
                     * u.getContent_type_field() + "&name=" + u.getFile_name_field(); logger.debug("setting upload link
                     * to " + href); uploadLinkNode.setHref(href);
                     */
                    String c = "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName() + UPLOAD_LINK_POSTFIX;
                    logger.debug("setting upload link class to " + c);
                    uploadLinkNode.setAttribute("class", c);
                }
                insertIntoDOM(parent, uploadNode.cloneNode(true), isInsertBefore, insert);
            } else if (ele.getTextarea() != null) {
                if (logger.isDebugEnabled()) logger.debug("creating a textarea element");
                Textarea t = ele.getTextarea();
                if (textareaNameNode != null) textareaNameNode.getFirstChild().setNodeValue(ele.getName());
                textareaLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                textareaInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                if (t.getRows() != null)
                    textareaInputNode.setRows(Integer.parseInt(t.getRows()));
                else
                    textareaInputNode.removeAttribute("rows");
                if (t.getCols() != null)
                    textareaInputNode.setCols(Integer.parseInt(t.getCols()));
                else
                    textareaInputNode.removeAttribute("cols");
                insertIntoDOM(parent, textareaNode.cloneNode(true), isInsertBefore, insert);
            } else if (ele.getSelect() != null) {
                if (logger.isDebugEnabled()) logger.debug("creating a select element");
                if (selectNameNode != null) selectNameNode.getFirstChild().setNodeValue(ele.getName());
                selectLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                selectInputNode.setAttribute("class", "Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                logger.debug("Dir::Get_Data." + xmlForm.getName() + "." + ele.getName());
                insertIntoDOM(parent, selectNode.cloneNode(true), isInsertBefore, insert);
            } else if (ele.getRadio() != null) {
                logger.debug("creating a radio button element");
                throw new RuntimeException("not yet implemented");
            } else if (ele.getCheckbox() != null) {
                logger.debug("createing checkbox elements");
                String modelName = xmlForm.getName() + "_" + ele.getName();
                checkboxInputNode.setAttribute("class", "Dir::Get_Data." + modelName + ".KEY");
                checkboxValueLabelNode.setAttribute("class", "Dir::Get_Data." + modelName + ".VALUE");
                checkboxLabelNode.getFirstChild().setNodeValue(ele.getLabel());
                checkboxGroupNode.setAttribute("class", "Dir::Iterate_Start." + modelName + " Dir::Iterate_Next."
                        + modelName);
                HTMLElement clone = (HTMLElement) checkboxGroupNode.cloneNode(false);
                clone.setAttribute("class", "Dir::Iterate_End." + modelName + " Dir::Discard");
                checkboxGroupNode.getParentNode().appendChild(clone);
                insertIntoDOM(parent, checkboxNode.cloneNode(true), isInsertBefore, insert);
            }

        }
        if (textNode != null) parent.removeChild(textNode);
        if (selectNode != null) parent.removeChild(selectNode);
        if (textareaNode != null) parent.removeChild(textareaNode);
        if (uploadNode != null) parent.removeChild(uploadNode);
        if (radioNode != null) parent.removeChild(radioNode);
        if (checkboxNode != null) parent.removeChild(checkboxNode);

        if (logger.isDebugEnabled()) {
            // trace the whole page
            StringWriter sw = new StringWriter();
            HTMLSerializer serializer = new HTMLSerializer(sw, new OutputFormat());
            try {
                serializer.serialize(page);
            } catch (IOException e) {
                logger.error(e.getMessage(), e);
            }
            logger.debug("page: " + sw.toString());
        }
    }

}