/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AbstractFormValidator.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import org.apache.log4j.Logger;

/**
 * The root implementation of the FormValidator interface
 */
public abstract class AbstractFormValidator implements FormValidator {

    protected static final Logger localLogger = Logger.getLogger(AbstractFormValidator.class.getName());

    private String myErrMsg = null;

    /**
     * Set the error message to be used in the event of an error.
     *
     * @param ierrmsg the error message to be used in the event
     *        of an error
     */
    public void setErrorMessage(String ierrmsg) {
        this.myErrMsg = ierrmsg;
    }

    /**
     * Get the error message to be used in the event of an error.
     *
     * @return the error message to be used in the event
     *        of an error
     */
    public String getErrorMessage() {
        return this.myErrMsg;
    }

    /**
     * This is a simple convenience method to return the appropriate type
     * of exception based on whether we're deferring or not.
     *
     * @param source     the source of the exception
     * @param deferExceptions
     *                   true if we want to generate a DeferredValidationException
     * @param defaultMsg the error message to use if no error message specified in
     *                   the constructor of the validator
     * @return a validation exception 
     */
    public ValidationException generateException(Object source, boolean deferExceptions, String defaultMsg) {
        if (localLogger.isDebugEnabled()) localLogger.debug("Generating exception... defer="+deferExceptions);
        String errmsg = (null==this.getErrorMessage()) ? defaultMsg : this.getErrorMessage();
        if (deferExceptions) return new DeferredValidationException(source, errmsg);
        else return new ValidationException(source, errmsg);
    }

    // ilc_02212002 start
    // added new method to check for nullness
    /**
     * Check if val passed is null in a consistant manner.
     * Most validators should call isNull(val) first thing
     * and return if that is the case.  Leave null validation
     * up to NotNullValidator.
     *
     * @param val the value to test for nullness
     */
    public boolean isNull(Object val, FormElement element){
      return ((val==null) || (val.toString().trim().length() < 1));
    }
    // ilc_02212001 End

}
