/*
 * Copyright (C) 2003  Chris Webb <chris.webb@voxsurf.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: FormType.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.*;
import java.util.Date;
import java.util.Locale;

import org.barracudamvc.plankton.StringUtil;

/**
 * This class defines all valid FormTypes. Currently we support:
 *
 * <ul>
 *        <li>String</li>
 *        <li>Boolean</li>
 *        <li>Integer</li>
 *        <li>Long</li>
 *        <li>Short</li>
 *        <li>Double</li>
 *        <li>Float</li>
 *        <li>BigDecimal</li>
 *        <li>Date</li>
 *        <li>Timestamp</li>
 *        <li>Time</li>
 * </ul>
 * @author  Chris Webb <chris.webb@voxsurf.com>
 * @author  Diez Roggisch <diez.roggisch@artnology.com>
 * @author  Iman L. Crawford <icrawford@greatnation.com>
 * @author  Christian Cryder <christianc@granitepeaks.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0
 */
public abstract class FormType {

    public static final Locale DEFAULT_LOCALE = Locale.getDefault();

    // String type definition.
    public static FormType STRING = new FormType() {
        public Class getFormClass() {
            return String.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            return origVal;
        }

        public Object [] getTypeArray(int size) {
            return new String [size];
        }
    };

    // Boolean type definition.
    public static FormType BOOLEAN = new FormType() {
        public Class getFormClass() {
            return Boolean.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            String tval = origVal.trim().toLowerCase();
            //csc_061902.1            if (tval.equals("on") || tval.equals("yes") || tval.equals("true")) {
            if (tval.equals("on") || tval.equals("yes") || tval.equals("true") || tval.equals("y")) {   //csc_061902.1
                origVal = "true";
                //csc_061902.1            } else if (tval.equals("off") || tval.equals("no") || tval.equals("false")) {
            } else if (tval.equals("off") || tval.equals("no") || tval.equals("false") || tval.equals("n")) {   //csc_061902.1
                origVal = "false";
            } else {
                throw new ParseException(origVal);
            }
            return new Boolean(origVal);
        }

        public Object [] getTypeArray(int size) {
            return new Boolean [size];
        }
    };

    // Integer type definition.
    public static FormType INTEGER = new FormType() {
        public Class getFormClass() {
            return Integer.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            try {
                return new Integer(origVal);
            } catch (NumberFormatException e) {
                try {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal);
                    int d1val = d1.intValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        return new Integer(d1val);
                    } else {
                        throw e;
                    }
                } catch (NumberFormatException e2) {
                    throw new ParseException(e2);
                }
            }
        }
        public Object [] getTypeArray(int size) {
            return new Integer [size];
        }
    };

    // Long type definition.
    public static FormType LONG = new FormType() {
        public Class getFormClass() {
            return Long.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            // ilc_022702.1_start
            // check for 0's after decimal place.
            // val = new Long(origVal);
            try {
                return new Long(origVal);
            } catch (NumberFormatException e) {
                try {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal);
                    long d1val = d1.longValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        return new Long(d1val);
                    } else {
                        throw e;
                    }
                } catch (NumberFormatException e2) {
                    throw new ParseException(e2);
                }
            }
            // ilc_022702.1_end
        }

        public Object [] getTypeArray(int size) {
            return new Long [size];
        }
    };

    // Short type definition.
    public static FormType SHORT = new FormType() {
        public Class getFormClass() {
            return Short.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            // ilc_022702.2_start
            // check for 0's after decimal place.
            // val = new Short(origVal);
            try {
                return new Short(origVal);
            } catch (NumberFormatException e) {
                try {
                    //this basically handles the case where the user
                    //typed in an integer value like 123.00...we convert
                    //that value to a double, then to an int, and then form
                    //a new double from that. If the values are equal, we
                    //know no roundoff occurred, meaning the decimal places
                    //were all zeros, and we're in business. Otherwise, throw
                    //the exception
                    Double d1 = new Double(origVal);
                    short d1val = d1.shortValue();
                    Double d2 = new Double(d1val);
                    if (d1.equals(d2)) {
                        return new Short(d1val);
                    } else {
                        throw e;
                    }
                } catch (NumberFormatException e2) {
                    throw new ParseException(e2);
                }
            }
            // ilc_022702.2_end
        }
        public Object [] getTypeArray(int size) {
            return new Short [size];
        }
    };

    // Double type definition.
    public static FormType DOUBLE = new FormType() {
        public Class getFormClass() {
            return Double.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            try {
                return new Double(origVal);
            } catch (NumberFormatException e) {
                throw new ParseException(e);
            }
        }

        public Object [] getTypeArray(int size) {
            return new Double [size];
        }
    };

    // Float type definition.
    public static FormType FLOAT = new FormType() {
        public Class getFormClass() {
            return Float.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null)
                return null;

            try {
                return new Float(origVal);
            } catch (NumberFormatException e) {
                throw new ParseException(e);
            }
        }

        public Object [] getTypeArray(int size) {
            return new Float [size];
        }
    };

    // BigDecimal type definition.
    public static FormType BIG_DECIMAL = new FormType() {
        public Class getFormClass() {
            return BigDecimal.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            //csc_060702.1_start - I changed this to use the String constructor since
            //the javadocs say that using the Double constructor is unpredicatable. Note
            //that I also am stripping out the dollar sign if its there (probably still
            //need to handle other currency symbols, based on locales)
            /*
              Object d = DOUBLE.parse(origVal, locale);
              if (d!=null) {
              return new BigDecimal(((Double)d).doubleValue());
              }
              return null;
            */
            //csc_040203.2 - wrapped the logic in a block to catch NumberFormatExceptions...not sure why this hadn't been done originally!
            try {
                String s = origVal.trim();
                s = StringUtil.replace(s, "$","");  //strip off $ sign
                s = StringUtil.replace(s, "�","");  //strip off � sign
                s = StringUtil.replace(s, ",","");  //strip off commas
                if (s.startsWith("(") && s.endsWith(")")) { //if its a debit (ie. in parenthesis), strip off parenthesis and add a - sign
                    s = "-"+s.substring(1,s.length()-1);
                }
                return new BigDecimal(s);
            } catch (NumberFormatException e) {
                throw new ParseException(e);
            }


            //csc_060702.1_end
        }

        public Object [] getTypeArray(int size) {
            return new BigDecimal [size];
        }
    };

    // Date type definition. Note that we DO not try and parse as SQL Dates, since these do not correctly handle locale info
    public static FormType DATE = new FormType() {
        public Class getFormClass() {
            return java.util.Date.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            // Eliminate the obvious.
            if (origVal==null) return null;

//csc_101504_1_start
            if (locale==null) locale = Locale.getDefault();

//csc_102405_2 - this causes problems in that a user entered date like 010105 will get interpreted as a millisecs value rather than a
//csc_102405_2   standard date. Since we can't find any instances of existing code actually relying on the Long behavior, go ahead 
//csc_102405_2   and rmove it (it's a nice idea, but it shouldn't co-opt a user entered date like 010105).
//csc_102405_2            //first try to convert from a long value
//csc_102405_2            try {
//csc_102405_2                return new Date(Long.parseLong(origVal));
//csc_102405_2            } catch (NumberFormatException e0) {
                // try all the various forms of date format to cover more bases
                try {
                    DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, locale);
                    df.setLenient(false);
                    return df.parse(origVal);
                } catch (java.text.ParseException e1) {
                    try {
                        DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM, locale);
                        df.setLenient(false);
                        return df.parse(origVal);
                    } catch (java.text.ParseException e2) {
                        try {
                            DateFormat df = DateFormat.getDateInstance(DateFormat.LONG, locale);
                            df.setLenient(false);
                            return df.parse(origVal);
                        } catch (java.text.ParseException e3) {
                            try {
                                DateFormat df = DateFormat.getDateInstance(DateFormat.FULL, locale);
                                df.setLenient(false);
                                return df.parse(origVal);
                            } catch (java.text.ParseException e4) {
                                throw new ParseException(e4, "Locale is " + locale.getCountry());
                            }
                        }
                    }
                }
//csc_102405_2            }
        }

        public Object[] getTypeArray(int size) {
            return new Date[size];
        }
    };

    //csc_051503_1 - added
    // Timestamp type definition.
    public static FormType TIMESTAMP = new FormType() {
        public Class getFormClass() {
            return Timestamp.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            //saw_061203_2 begin - TIMESTAMP should work much the same way as DATE, only it should
            //use DateFormat.getDateTimeInstance() instead of getDateInstance().
/*
            Timestamp ts = null;
            Long l = null;

            //first of all, try and see if we can treat it as a Long value
            try {
                l = (Long) LONG.parse(origVal, locale);
            } catch (ParseException e) {}

            //if that didn't work, try to convert it as a date
            if (l==null) {
                Date d = (Date) DATE.parse(origVal, locale);
                l = new Long(d.getTime());
            }

            //finally, try creating the timestamp from the long value
            return new Timestamp(l.longValue());
*/
            if (origVal==null) return null;
            if (locale==null) locale = Locale.getDefault();

//csc_102405_2 - this causes problems in that a user entered date like 010105 will get interpreted as a millisecs value rather than a
//csc_102405_2   standard date. Since we can't find any instances of existing code actually relying on the Long behavior, go ahead 
//csc_102405_2   and rmove it (it's a nice idea, but it shouldn't co-opt a user entered date like 010105).
//csc_102405_2            //first try to parse from a long value
//csc_102405_2            try {
//csc_102405_2                return new Timestamp(Long.parseLong(origVal));
//csc_102405_2            } catch (NumberFormatException e0) {
                // try all the various forms of datetime format to cover more bases
                try {
                    DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, locale);
                    df.setLenient(false);
                    return new Timestamp(df.parse(origVal).getTime());
                } catch (java.text.ParseException e1) {
                    try {
                        DateFormat df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, locale);
                        df.setLenient(false);
                        return new Timestamp(df.parse(origVal).getTime());
                    } catch (java.text.ParseException e2) {
                        try {
                            DateFormat df = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG, locale);
                            df.setLenient(false);
                            return new Timestamp(df.parse(origVal).getTime());
                        } catch (java.text.ParseException e3) {
                            try {
                                DateFormat df = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL, locale);
                                df.setLenient(false);
                                return new Timestamp(df.parse(origVal).getTime());
                            } catch (java.text.ParseException e4) {
                                //finally try the SQL timestamp format
                                try {
                                    return Timestamp.valueOf(origVal);
                                } catch (IllegalArgumentException e5) {
                                    throw new ParseException(e5, "Locale is "+locale.getCountry());
                                }
                            }
                        }
                    }
//csc_102405_2                }
            }
            //saw_061203_2 end
        }

        public Object [] getTypeArray(int size) {
            return new Timestamp[size];
        }
    };

    /**
     * Time type definition
     * @since saw_061203_2
     * @clientCardinality 1
     */
    public static FormType TIME = new FormType() {
        public Class getFormClass() {
            return Time.class;
        }

        public Object parse(String origVal, Locale locale) throws ParseException {
            if (origVal==null) return null;
            if (locale==null) locale = Locale.getDefault();

//csc_102405_2 - this causes problems in that a user entered date like 010105 will get interpreted as a millisecs value rather than a
//csc_102405_2   standard date. Since we can't find any instances of existing code actually relying on the Long behavior, go ahead 
//csc_102405_2   and rmove it (it's a nice idea, but it shouldn't co-opt a user entered date like 010105).
//csc_102405_2            //first try to parse from a long value
//csc_102405_2            try {
//csc_102405_2                return new Time(Long.parseLong(origVal));
//csc_102405_2            } catch (NumberFormatException e0) {
                // try all the various forms of time format to cover more bases
                try {
                    DateFormat df = DateFormat.getTimeInstance(DateFormat.SHORT, locale);
                    df.setLenient(false);
                    return new Time(df.parse(origVal).getTime());
                } catch (java.text.ParseException e1) {
                    try {
                        DateFormat df = DateFormat.getTimeInstance(DateFormat.MEDIUM, locale);
                        df.setLenient(false);
                        return new Time(df.parse(origVal).getTime());
                    } catch (java.text.ParseException e2) {
                        try {
                            DateFormat df = DateFormat.getTimeInstance(DateFormat.LONG, locale);
                            df.setLenient(false);
                            return new Time(df.parse(origVal).getTime());
                        } catch (java.text.ParseException e3) {
                            try {
                                DateFormat df = DateFormat.getTimeInstance(DateFormat.FULL, locale);
                                df.setLenient(false);
                                return new Time(df.parse(origVal).getTime());
                            } catch (java.text.ParseException e4) {
                                //finally try the SQL time format
                                try {
                                    return Time.valueOf(origVal);
                                } catch (IllegalArgumentException e5) {
                                    throw new ParseException(e5, "Locale is "+locale.getCountry());
                                }
                            }
                        }
                    }
                }
//csc_102405_2            }
            //saw_061203_2 end
        }

        public Object [] getTypeArray(int size) {
            return new Time[size];
        }
    };

    /**
     * Protected constructor to prevent external instantiation. Cannot be
     * private because we would be unable to call the constructor from a
     * sub-class.
     */
    protected FormType() { }

    /**
     * Returns the class associated with this particular form type.
     */
    public abstract Class getFormClass();

    /**
     * Parses an object based on the specific form type.
     */
    public Object parse(String origVal) throws ParseException {
        return parse(origVal, null);
    }

    /**
     * Parses an object based on the specific form type.
     */
    public abstract Object parse(String origVal, Locale loc) throws ParseException;


    /** create an array of the FormType's type - if heterogenous types
     * are returned, an array of Object will be returned.
     */
    public abstract Object [] getTypeArray(int size);

    /**
     * Returns a string representation of this particular formt type.
     */
    public String toString() {
        return this.getFormClass().getName();
    }

}
