/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DefaultServletRequestWrapper.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.helper.servlet;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.log4j.Logger;
import org.barracudamvc.core.helper.state.ParamPersister;
import org.barracudamvc.plankton.data.Param;

//csc_010404_1 - revamped to extend from servlet package's HttpServletRequestWrapper
/**
 * <p>This class acts as a thin wrapper around a ServletRequest. Most calls
 * are simply passed through to the underlying request object. This object
 * does however, expose a method which allows you to set parameters in
 * the request object. This was necessary for cases where we needed to
 * be able to do a POST, save the parameters somewhere, and then do a GET
 * and reconstitute the parameters from that.
 *
 * <p>When you instantiate this object, it will automatically check the 
 * clients session to see if there are any parameter state information that
 * needs to be reconstituted into the current request.
 */
public class DefaultServletRequestWrapper extends HttpServletRequestWrapper implements BarracudaServletRequestWrapper {

    protected static final Logger logger = Logger.getLogger(DefaultServletRequestWrapper.class.getName());

//csc_041905_1    public static String DEFAULT_ENCODING = "UTF-8";    //csc_041805_1
    public static String DEFAULT_ENCODING = System.getProperty("file.encoding");    //csc_041905_1

    HttpServletRequest req = null;
    List paramList = null;
    String method = null;
    
    /**
     * Create a DefaultServletRequestWrapper around some other
     * HttpServletRequest impl. The wrapper adds the ability to
     * add/remove parameter values.
     *
     * @param ireq the underlying HttpServletRequest
     */
    public DefaultServletRequestWrapper(HttpServletRequest ireq) {
        super(ireq);
        setRequest(ireq);
    }


    //--------------- BarracudaServletRequestWrapper -------------
    /**
     * Set the underlying request method.
     *
     * @param method the underlying request method (GET, POST, etc)
     */
    public void setMethod(String imethod) {
        method = imethod;        
    }

    /**
     * Get the underlying request method.
     */
    public String getMethod() {
        return (method!=null ? method : req.getMethod());        
    }

    /**
     * Set a given parameter (note that this is backed by a hashmap,
     * so the structure is slightly different than that of the
     * underlying ServletRequest which allows multiple paramters
     * with the same name). This means that if you attempt to
     * set a parameter whose key already exists you will effectively
     * overwrite the existing value.
     *
     * @param name the key name for the parameter
     * @param value the value associated with the given key
     */
    public void addParameter(String name, String value) {
        //eliminate the obvious
        if (name==null) return;

        //make sure the paramList is initialized
        if (paramList==null) setupParamList();

        //finally store the new value
        paramList.add(new Param(name, value));
    }

    /**
     * Returns the value of a request parameter as a String, or
     * null if the parameter does not exist.
     *
     * @param name the key name for the parameter
     * @return the parameter value associated with a key name
     */
    public String getParameter(String name) {
        //eliminate the obvious
        if (name==null) return null;

        //if paramList exists, get the value from there
        if (paramList!=null) {
            Iterator it = paramList.iterator();
            while (it.hasNext()) {
                Param param = (Param) it.next();
                if (param.getKey().equals(name)) return param.getValue();
            }
            return null;

        //otherwise just delegate to the underlying request
        } else {
//merg_092901.1_start
            //This patch submitted by Merg [merg@merg.be]. The basic problem 
            //is that some servlet containers do not accurately report all 
            //parameters submitted with the request. If the request is a post, 
            //and there were additional parameters submitted in the url, the 
            //additional params are sometimes not returned (ie. on ATG Dynamo). 
            //SO...if the value comes back null, then actually check the
            //query string for them.
//          return req.getParameter(name);
            String s = req.getParameter(name);
            if (s==null) {
                // Still no parameter found, check the queryString
                String queryString = req.getQueryString();
                if (queryString!=null) {
//csc_110102.1_start - fix deprecation issues                
//csc_110402.1 - revert
//jrk_20030731.1_start - catch runtime exceptions in the decode() method
                    try {
//csc_041805_1                        queryString = java.net.URLDecoder.decode(queryString);
                        queryString = java.net.URLDecoder.decode(queryString, DEFAULT_ENCODING);  //csc_041805_1
                    } catch (Exception e) {
                        logger.warn("failed to decode queryString, but allowing to continue", e);
                    }
//jrk_20030731.1_end
/*
                    try {
                        queryString = java.net.URLDecoder.decode(queryString, "UTF-8");
                    } catch (UnsupportedEncodingException e) {
                        System.out.println("Encoding Exception: "+e);
                        e.printStackTrace();
                    }
*/                    
//csc_110102.1_end
//jrk_20030828.1_start
//                    int startPos = queryString.indexOf(name + "="); //need "=" to know it is a parameter name as opposed to a value
//                    int endPos = -1;

                    //need "=" to know it is a parameter name as opposed to a value
                    // also need '&' to make difference between id and aaa_id for example
                    // if is first param, will be checked in different place
                    int startPos = queryString.indexOf("&"+name + "="); 
                    int endPos = -1;

                    if (startPos == -1) {
                        // not found  inside parameter list, try find as first
                        if (queryString.startsWith(name+"=")) {
                            startPos = 0;
                        }
                    } else {
                        // skip '&'
                        startPos++;
                    }
//jrk_20030828.1_end

                    if (startPos!=-1) {
                        startPos = startPos + name.length() + 1;
                        endPos = queryString.indexOf("&", startPos);

                        if (endPos==-1) {
                            s = queryString.substring(startPos);
                        }
                        else {
                            s = queryString.substring(startPos, endPos);
                        }
                    }
                }
            }
            return s;
//merg_092901.1_end
        }
    }

    /**
     * Returns an Enumeration of String objects containing the
     * names of the parameters contained in this request.
     *
     * @return an Enumeration of all the parameter names
     */
    public Enumeration getParameterNames() {
        //if paramList is not null, get the enum from there
        if (paramList!=null) {
            return new LocalEnumerator(paramList);

        //otherwise just delegate to the underlying request
        } else {
            return req.getParameterNames();
        }
    }

    /**
     * Returns a java.util.Map of the parameters of this request. 
     * Request parameters are extra information sent with the request. 
     * For HTTP servlets, parameters are contained in the query string 
     * or posted form data.
     */
    public Map getParameterMap() {
        //if paramList exists, get the value from there
        if (paramList!=null) {
            Iterator it = paramList.iterator();
            Map paramMap = new HashMap(paramList.size());
//            Map paramMap = new TreeMap();
//csc_120602.1_start
/*
Ok, so the problem with this is that 
a) the values should be stored as a String[], and 
b) if you have multiple values for a given its not going to work as implemented
            while (it.hasNext()) {
                Param param = (Param) it.next();
                paramMap.put(param.getKey(), param.getValue());
*/                
            //populate the paramMap with key/val pairs
            while (it.hasNext()) {
                Param param = (Param) it.next();
                String key = param.getKey();
                List valList = (List) paramMap.get(key);
                if (valList==null) {
                    valList = new ArrayList(10);
                    paramMap.put(key, valList);
                }
                valList.add(param.getValue());
            }

            //now run back through the paramMap and convert all the 
            //List values into String[] (to conform with servlet spec)
            it = paramMap.keySet().iterator();
            while (it.hasNext()) {
                Object key = it.next();
                List valList = (List) paramMap.get(key);
//csc_010404_1_start
//Not sure why, but all of a sudden this code was erring with a Class cast exception - 
//List seems to return an Object[] and that can't be cast to a String[] even if all the items
//in the list are Strings
//                paramMap.put(key, (String[]) valList.toArray());
                int idx = -1;
                String[] valArr = new String[valList.size()];
                Iterator it2 = valList.iterator();
                while (it2.hasNext()) {
                    valArr[++idx] = (String) it2.next();
                }
                paramMap.put(key, valArr);
//csc_010404_1_end
            }
//csc_120602.1_end

            return paramMap;

        //otherwise just delegate to the underlying request
        } else {
            return req.getParameterMap();
        }
    }
    //csc_013102.1_end

    /**
     * Returns an array of String objects containing all of the
     * values the given request parameter has, or null if the
     * parameter does not exist.
     *
     * @param name the key name for the parameter
     * @return an array of Strings for the given key name
     */
    public String[] getParameterValues(String name) {
        //eliminate the obvious
        if (name==null) return null;

        //if paramList is not null, build the array from there
        if (paramList!=null) {
            List valueList = new ArrayList(paramList.size());
            Iterator it = paramList.iterator();
            while (it.hasNext()) {
                Param param = (Param) it.next();
                if (param.getKey().equals(name)) valueList.add(param.getValue());
            }
            int idx = -1;
            String[] valueArr = new String[valueList.size()];
            it = valueList.iterator();
            while (it.hasNext()) {
                valueArr[++idx] = (String) it.next();
            }
            if (valueArr.length==0) return null;
            else return valueArr;

        //otherwise just delegate to the underlying request
        } else {
            return req.getParameterValues(name);
        }
    }

    /**
     * Remove the first parameter whose key matches the specified name
     *
     * @param name the key name for the parameter
     */
    public void removeParameter(String name) {
        //eliminate the obvious
        if (name==null) return;

        //make sure the paramList is initialized
        if (paramList==null) setupParamList();

        //finally remove the first occurence of the parameter
        for (int i=0, max=paramList.size(); i<max; i++) {
            Param param = (Param) paramList.get(i);
            if (param.getKey().equals(name)) {
                paramList.remove(i);
                break;
            }
        }

    }

    /**
     * Remove all parameters for a specified name
     *
     * @param name the key name for the parameter
     */
    public void removeAllParameters(String name) {
        //eliminate the obvious
        if (name==null) return;

        //make sure the paramList is initialized
        if (paramList==null) setupParamList();

        //finally remove the all occurences of the parameter
        for (int i=paramList.size()-1; i>=0; i--) {
            Param param = (Param) paramList.get(i);
            if (param.getKey().equals(name)) paramList.remove(i);
        }
    }

    /**
     * Reset the parameter values to their original state
     * (ie. the actual values in the request)
     */
    public void resetParameters() {
        paramList=null;
    }

    /**
     * Get the underlying servlet request. The only reason you
     * should ever have to do this is if you are trying to forward
     * a request. Some containers check to make sure that the
     * request object being forwarded is an instance of their own
     * implementation...
     *
     * @return the underlying servlet request object
     */
    public HttpServletRequest getCoreRequest() {
        return req;
    }


    //--------------- ServletRequestWrapper ----------------------
    /**
     * Set the underlying request object.
     *
     * @param ireq the underlying HttpServletRequest
     */
    public void setRequest(ServletRequest ireq) {
        super.setRequest(ireq);

        if (req!=ireq && ireq instanceof HttpServletRequest) {                  
            //set the reference to the req
            req = (HttpServletRequest) ireq;
        
            //reconstitute any param values from the user's session 
            ParamPersister.reconstituteReqParamState(this);
        }
    }




    //--------------- HttpServletRequestWrapper ------------------
    




    //-------------------- Utility stuff -------------------------
    private void setupParamList() {
        //eliminate the obvious (only initialize once!)
        if (paramList!=null) return;

        //create the param list
        paramList = new ArrayList(10);

        //now copy in all param values from the underlying servlet
        //request. From this point on then, the param values will
        //be maintained in the paramList
        Enumeration enumeration = req.getParameterNames();
        while (enumeration.hasMoreElements()) {
            //get the key
            String key = (String) enumeration.nextElement();

            //find all values associated with the key
            String[] vals = req.getParameterValues(key);
            for (int i=0, max=vals.length; i<max; i++) {
                paramList.add(new Param(key, vals[i]));
            }
        }
    }

    /**
     * This inner class implements Enumaration. It will effectively
     * enumerate over all of the parameter key names.
     */
    class LocalEnumerator implements Enumeration {
        List keyList = null;
        Iterator it = null;

        public LocalEnumerator(List iparamList) {
            keyList = new ArrayList(iparamList.size());
            it = iparamList.iterator();
            while (it.hasNext()) {
                Param param = (Param) it.next();
                if (!keyList.contains(param.getKey())) keyList.add(param.getKey());
            }
            it = keyList.iterator();
        }

        public boolean hasMoreElements() {
            return (it.hasNext());
        }

        public Object nextElement() {
            return it.next();
        }
    }
}