/*
 * Copyright (C) 2003 Stefan Armbruster
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MultipartRequestWrapper.java 124 2005-12-28 06:03:02Z jkjome $
 */
package org.barracudamvc.core.helper.servlet;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.log4j.Logger;

//saw_060105_1 - Taken from org.barracudamvc.contrib.sam.servlet.FileUploadRequestWrapper and modified
//jrk_122705_1 - updated docs, added constructor to take a FileItemFactory for easy configuration, fixed getParameter* methods to only return form field param info, not file param info, added getFileParameterNames() to get only parameters names associated with file fields, added isMultipart() to be able to distinguish if the current request is truly multipart.
/**
 * Handles both simple and multipart requests transparently
 *
 * <p>For handling multipart stuff, Jakarta Commons Fileupload is used.
 * Otherwise this wrapper's behavior is identical to the superclass.
 * getFileItem* methods are provided to get direct access to file
 * items in a multipart request and an isMultipart() method is provided for
 * querying whether the current request has multipart data in it or not. The
 * only drawback to using this wrapper is the slight overhead involved in doing
 * multipart checks when the request isn't multipart. The performance deficit
 * should, however, be neglegible.</p>
 *
 * @author  Stefan Armbruster (sam, 2003-11-15)
 * @author  Shawn A. Wilson
 * @author  Jacob Kjome
 */
public class MultipartRequestWrapper extends DefaultServletRequestWrapper {

    private static final Logger logger = Logger.getLogger(MultipartRequestWrapper.class);

    /** maximum allowed size of uploads */
    protected long maxSize = -1; //TODO - remove this as a field after determining it won't hurt anyone.  Of course, it wouldn't be an issue if fields were private in the first place!
    
    /** indicates if the current request is multipart */
    protected boolean isMultipart;

    /** contains a list of {@link FileItem file items} */
    protected List uploadItemList;

    /**
     * Checks if request is multipart. If so, initializes upload items.
     * 
     * @param req the request
     * @param max the maximum allowed size of uploads, -1 signifies no limit
     * @param fileItemFactory some FileItemFactory implementation, defaults to DiskFileItemFactory if null 
     */
    public MultipartRequestWrapper(HttpServletRequest req, long max, FileItemFactory fileItemFactory) {
    	super(req);
    	this.uploadItemList = new ArrayList();
        this.maxSize = max;
        //Note: ServletFileUpload.isMultipartContent(HttpServletRequest) is marked as deprecated in commons-fileupload-1.1 only as a consequence of maintaining backward compatibility.  It will be removed from FileUploadBase and moved to ServletFileUpload in a future version. Using the future-compatible call to avoid breakage and, eventually, get rid of deprecation warning.
        this.isMultipart = ServletFileUpload.isMultipartContent(req);
        if (logger.isDebugEnabled()) logger.debug("request is multipart: " + isMultipart);

        if (this.isMultipart) {
            initUploadItemList(req, fileItemFactory != null ? fileItemFactory : new DiskFileItemFactory());
        }
    }
    
    /** 
     * @param req the request
     * @param max Maximum allowed size of uploads
     * @see #this(HttpServletRequest, long, FileItemFactory)
     */
    public MultipartRequestWrapper(HttpServletRequest req, long max) {
        this(req, max, null);
    }

    /**
     * Simple constructor, accepts any file size and uses the default
     * FileItemFactory
     * 
     * @param req
     * @see #this(HttpServletRequest, long, FileItemFactory)
     */
    public MultipartRequestWrapper(HttpServletRequest req) {
        this(req, -1, null);
    }

    /**
     * Parses the multipart request for upload items and stores them as
     * {@link FileItem file items} in a list
     * 
     * @param req the request
     * @param fileItemFactory the FileItemFactory
     * @see ServletFileUpload
     */
    private void initUploadItemList(HttpServletRequest req, FileItemFactory fileItemFactory) {
        ServletFileUpload upload = new ServletFileUpload(fileItemFactory);

        if (maxSize != -1)
            upload.setSizeMax(maxSize);
        try {
            uploadItemList = upload.parseRequest(req);
            logger.debug("successfully parsed request");
        } catch (FileUploadException e) {
            logger.error(e.getMessage(), e);
        }
    }

    /**
     * Same behavior as ServletRequest.getParameter, but extended to handle
     * parameters when request is multipart. Only non-file parameters are
     * applicable here.
     * 
     * @param name a String specifying the name of the parameter
     * @return a String representing the single value of the parameter
     * @see javax.servlet.ServletRequest#getParameter(String)
     */
    public String getParameter(String name) {
        if (name == null) return null;

        if (isMultipart()) {
            return getUploadItemValueByName(name);
        }

        return super.getParameter(name);
    }

    /**
     * Same behavior as ServletRequest.getParameterNames, but extended to handle
     * parameter names when request is multipart. Only parameter names representing
     * non-file parameters are applicable here.
     * 
     * @return an Enumeration of String objects, each String containing the name of a request parameter; or an empty Enumeration if the request has no parameters
     * @see javax.servlet.ServletRequest#getParameterNames()
     */
    public Enumeration getParameterNames() {
        if (isMultipart()) {
        	List allNames = new ArrayList();
            for (Iterator iter = uploadItemList.iterator(); iter.hasNext();) {
                FileItem item = (FileItem) iter.next();
                String fieldName = item.getFieldName();

                if (logger.isDebugEnabled()) logger.debug(
                    "item "
                    + "fieldname: "
                    + fieldName
                    + ", name: "
                    + item.getName()
                    + ", size: "
                    + item.getSize()
                    + ", string: "
                    + item.getString()
                    + ", cnttype: "
                    + item.getContentType()
                    + ", isFormField"
                    + item.isFormField());

                if (item.isFormField()) {
                    allNames.add(fieldName);
                }
            }
            return Collections.enumeration(allNames);
        }

        return super.getParameterNames();        
    }

    /**
     * Akin to getParameterNames(), except that only parameter names
     * representing file parameters are applicable here. If the request is not
     * multipart or it is but there are no file fields to speak of, an empty
     * Enumeration will be returned.
     * 
     * @return an Enumeration of String objects, each String containing the name of a file request parameter; or an empty Enumeration if the request has no file parameters
     * @see #getParameterNames
     */
    public Enumeration getFileParameterNames() {
    	List allNames = new ArrayList();
    	if (isMultipart()) {
    		for (Iterator iter = uploadItemList.iterator(); iter.hasNext();) {
                FileItem item = (FileItem) iter.next();
                if (!item.isFormField()) {
                	allNames.add(item.getFieldName());
                }
    		}
        }
    	return Collections.enumeration(allNames);
    }

    /**
     * Same behavior as ServletRequest.getParameterValues, but extended to handle
     * parameters when request is multipart. Only non-file parameters are
     * applicable here.
     * 
     * @param name a String containing the name of the parameter whose value is requested
     * @return an array of String objects containing the parameter's values
     * @see javax.servlet.ServletRequest#getParameterValues(String)
     */
    public String[] getParameterValues(String name) {
        if (name == null) return null;

        if (isMultipart()) {
            String ret = getUploadItemValueByName(name);
            return ret != null ? new String[] {ret} : null;
        }

        return super.getParameterValues(name);
    }

    /**
     * Same behavior as ServletRequest.getParameterMap, but extended to handle
     * parameters when request is multipart. Only non-file parameters are
     * applicable here.
     *
     * @return an immutable java.util.Map containing parameter names as keys and parameter values as map values. The keys in the parameter map are of type String. The values in the parameter map are of type String array
     * @see javax.servlet.ServletRequest#getParameterMap
     */
    public Map getParameterMap() {
    	if (isMultipart()) {
    	    Map map = new HashMap();
    	    for (Enumeration enumer = this.getParameterNames(); enumer.hasMoreElements();) {
    		    String name = (String) enumer.nextElement(); 
    		    map.put(name, this.getParameterValues(name));
            }
    	    return Collections.unmodifiableMap(map);
    	}

    	return super.getParameterMap();
    }

    /**
     * Finds a matching item in uploadItems and returns its value as a string.
     * Returns null if the 'name' parameter is null, the named file item doesn't
     * exist, the file item does exist but it isn't a simple form field (in which
     * case the value is not provided as parameter data, only as file data), or
     * the request is not multipart.
     * 
     * @param name a String specifying the name of the parameter
     * @return the result of FileItem.getString(), first trying the encoding of the request and falling back to the system default if that fails
     * @see #getFileItem(String)
     * @see FileItem#getString(String)
     */
    protected String getUploadItemValueByName(String name) {
    	if (name == null) return null;

    	FileItem item = getFileItem(name);
        if (item == null || !item.isFormField()) return null;

        try {        	
        	return item.getString(this.getCharacterEncoding());
        } catch (Exception e) {
        	return item.getString();
        }
    }

    /**
     * Finds a matching item in uploadItems and returns it. Returns null if
     * the 'name' parameter is null or the request is not multipart.
     * 
     * @param name a String specifying the name of the parameter
     * @return the matching FileItem
     */
    public FileItem getFileItem(String name) {
    	if (name == null) return null;
    	if (!isMultipart()) return null;

        for (Iterator iter = uploadItemList.iterator(); iter.hasNext();) {
            FileItem item = (FileItem) iter.next();
            if (name.equals(item.getFieldName())) {
                return item;
            }
        }

        if (logger.isDebugEnabled()) logger.debug("no fileItem for field " + name + " found.");
        return null;
    }

    /**
     * Specifies whether the current request is multipart
     * 
     * @return true if multipart, false if not
     */
    public boolean isMultipart() {
    	return isMultipart;
    }

}
