/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ResourceGateway.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.helper.servlet;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.ref.Reference;
import java.lang.ref.SoftReference;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.barracudamvc.plankton.data.ReferenceFactory;
import org.barracudamvc.plankton.http.ContextServices;


/**
 * <p>The purpose of this servlet is to look for a static resource
 * on the classpath and return it to the client
 */
public class ResourceGateway extends HttpServlet {

    //public vars...eventually, these should probably be final
    protected static final Logger logger = Logger.getLogger(ResourceGateway.class.getName());

    //these are intentionally non-final, so that you can programatically
    //change them if need be
    public static String EXT_RESOURCE_ID = "xlib";
    public static String RESOURCE_NOT_FOUND = "ResourceNotFound";
    
    protected static final long startup = System.currentTimeMillis(); //saw_031604_1


    //-------------------- ResourceGateway -----------------------
    /**
     * <p>Handle the default HttpRequest.
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //figure out what resource we're looking for
        //jrk_2002-12-02.1 - don't prepend "/" otherwise the context classloader won't find the resource
        //String resourceName = "/"+EXT_RESOURCE_ID + req.getPathInfo();
        String resourceName = EXT_RESOURCE_ID + req.getPathInfo();
        if (logger.isDebugEnabled()) logger.debug("Looking for static resource: "+resourceName);

        //get the object from cache
        Object resource = ContextServices.getObjectFromCache(this.getServletContext(), resourceName, new LocalReferenceFactory(resourceName));

        //check the results
        //...byte array
        if (resource instanceof LocalResource) {
            
          //saw_031604_1 begin
            // if the client sends an "If-Modified-Since" then we should compare it against the
            // startup time and send a 304 (Not Modified) response if it hasn't changed
            long lastTime = req.getDateHeader("If-Modified-Since");
            
            // divide by 1000 because milliseconds aren't transmitted via HTTP
            if (lastTime/1000 >= startup/1000) {
                logger.debug("Sending 302 (Not Modified) response");
                resp.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                return;
            }
          //saw_031604_1 end

            //csc_061202.1 - added
            //set the caching hdrs (this will allow the static resources to be cached by the browser)
            resp.setHeader("Cache-Control", "public");
//saw_031604_1            resp.setDateHeader("Last-Modified", System.currentTimeMillis());
            resp.setDateHeader("Last-Modified", startup); //saw_031604_1

            //set the content type
            if (logger.isDebugEnabled()) logger.debug("Got resource from cache");
            LocalResource lr = (LocalResource)resource;
            resp.setContentType(lr.contentType);
            if (logger.isDebugEnabled()) logger.debug("Set content-type: "+lr.contentType);

            //now return the requested document
            OutputStream out = resp.getOutputStream();
            out.write(lr.contentData);
            //out.flush();
            resp.flushBuffer(); //instead of out.flush for Servlet 2.2 compatibility (Craig McClanahan http://w4.metronet.com/~wjm/tomcat/2000/Nov/msg00174.html)
            if (logger.isDebugEnabled()) logger.debug("Wrote data: "+lr.contentData.length+" bytes");

        //...resource not found
        } else {
            if (logger.isDebugEnabled()) logger.debug("Resource not found");
            resp.sendError(HttpServletResponse.SC_NOT_FOUND, "Resource Not Found: "+resourceName);
        }
    }

    /**
     * Used to create a SoftReference to a specified resource
     */
    class LocalReferenceFactory implements ReferenceFactory {
        String resourceName = null;

        public LocalReferenceFactory(String iresourceName) {
            resourceName = iresourceName;
        }

        public Reference getObjectReference() {
            if (logger.isDebugEnabled()) logger.debug("Resource not in cache...will load from classpath");

            //figure out the content type (see: http://www.isi.edu/in-notes/iana/assignments/media-types/media-types for source list)
            String contentType = ResourceGateway.this.getServletContext().getMimeType(resourceName);
            if (contentType==null) contentType = "text/plain";

            //get the resource
            if (logger.isDebugEnabled()) logger.debug("Attempting to read: "+resourceName);

            Object result = null;
            byte[] dstbytes = new byte[10240];
            try {
                //read in the file and store it in a byte array
                //jrk_2002-12-02.2 - get the resource from the context classloader instead of just the local classloader
                //BufferedInputStream in = new BufferedInputStream(this.getClass().getResourceAsStream(resourceName));
                BufferedInputStream in = new BufferedInputStream(Thread.currentThread().getContextClassLoader().getResourceAsStream(resourceName));
                int offset = 0;
                int len = 1024;
                byte[] inbytes = new byte[len];
                while (true) {
                    int cnt = in.read(inbytes, 0, len);
                    if (cnt==-1) break;
                    if (offset+cnt>=dstbytes.length) {
                        byte[] newbytes = new byte[(int) (dstbytes.length+(len*5))];
                        System.arraycopy(dstbytes, 0, newbytes, 0, dstbytes.length);
                        dstbytes = newbytes;
                    }
                    System.arraycopy(inbytes, 0, dstbytes, offset, cnt);
                    offset+=cnt;
                }
                //merg_100201.1 - Added a close() statement after reading the resource
                //from the inputstream. I noticed that I couldn't redeploy the app when
                //it used the ResourceGateway. This simple .close() fixes it.
                in.close();

                //when we're done trim off the excess
                byte[] newbytes = new byte[offset];
                System.arraycopy(dstbytes, 0, newbytes, 0, offset);
                dstbytes = newbytes;

                //once we're done reading, create a local resource object
                if (logger.isDebugEnabled()) logger.debug("Success!");
                LocalResource lr = new LocalResource();
                lr.contentType = contentType;
                lr.contentData = dstbytes;
                result = lr;
            } catch (IOException e) {
                if (logger.isDebugEnabled()) logger.debug("Failure: "+e);
                result = RESOURCE_NOT_FOUND;
            }
            return new SoftReference(result);
        }

    }

    /**
     * The actual resource that gets cached
     */
    class LocalResource {
        public String contentType = null;
        public byte[] contentData = null;
    }


    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }

    //-------------------- Servlet -------------------------------
    /**
     * <p>Here's where we initialize the servlet.
     */
    public void init() throws ServletException {
        logger.info("initializing servlet");
    }

}
