/*
 * Copyright (C) 2001  Arno Schatz [arno@jivan.org]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: JivanDOMWriter.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.util.dom;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.jivan.html.document.DocumentFactory;
import org.jivan.html.document.DocumentManager;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLDocument;

/** 
 * This class is a specialized DOMWriter for Jivan. It calls the Jivan serializers
 * for output to a Outputstream or java.io.Writer.
 * 
 * The root node of the tree to be outputted must be either a Document or Element.
 * This means DocumentFragments are not supported. (other node types don't 
 * really make sense here)
 * 
 * After printout, the subtree is being repaired and ready to be used again.
 * @author Arno, arno@jivan.org
 */
public class JivanDOMWriter implements DOMWriter {

	protected static final Logger logger = Logger.getLogger(JivanDOMWriter.class.getName());

	// prevent the HTML pages from being cached by the browser
	protected boolean preventCaching = false;

	// how long can the page be cached by browser
    protected String contentType = null;                    //csc_012804_1
    protected String contentDisposition = null;             //csc_012804_1
    protected boolean leaveWriterOpen = false;              //csc_012804_1
	protected int maxAge = 0;

	/**
	* Createing a DOMWriter for outputting a DOM created by Jivan
	*/
	public JivanDOMWriter() {
		super();
        logger.info("Instantiating DOMWriter: "+this);
	}

//csc_012804_1_start
    /**
     * Set the content type (defaults to "text/html" or "text/xml" depending on the 
     * document type
     */
    public void setContentType(String icontentType) {
        contentType = icontentType;
    }

    /**
     * Get the content type
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * Set the content disposition (ie. "inline; filename=foo.txt", defaults to null)
     */
    public void setContentDisposition(String icontentDisposition) {
        contentDisposition = icontentDisposition;
    }

    /**
     * Get the content disposition
     */
    public String getContentDisposition() {
        return contentDisposition;
    }

    /**
     * Set true if we want to leave the writer open (ie. for multiple writes)
     */
    public void setLeaveWriterOpen(boolean val) {
        leaveWriterOpen = val;
    }

    /**
     * return true if the writer is configured to leave the output stream open
     */
    public boolean getLeaveWriterOpen() {
        return leaveWriterOpen;
    }

    /**
     * Prepare the response object
     *
     * @param node the DOM node to be written out
     * @param resp the HttpServletResponse object
     */
    public void prepareResponse(Node node, HttpServletResponse resp) throws IOException {
        //saw_031604_1 - set a default content type if none is specified
        if (getContentType()==null) setContentType((node instanceof HTMLDocument) ? "text/html" : "text/xml");
        
        //set the content type and disposition
        if (logger.isDebugEnabled()) logger.debug("Setting content type:"+contentType+" content disposition:"+contentDisposition);
        if (contentType!=null) resp.setContentType(contentType);
        if (contentDisposition!=null) resp.setHeader("Content-Disposition", contentDisposition);

        //if we need to prevent caching
        if (preventCaching) {
            //add the appropriate headers to the response
            if (logger.isDebugEnabled()) logger.debug("updating resp hdr to prevent caching");
            resp.setHeader("Pragma","no-cache");
            resp.setHeader("Cache-Control","no-cache");
            resp.setDateHeader("Expires", System.currentTimeMillis());

        //csc_061202.1 - added
        //otherwise explicitly give it a max-age (this will generally allow browsers like
        //IE to page back in history without reloading, but if the user actually revisits the
        //URL, then it will still be reloaded)
        } else {
            resp.setHeader("Cache-Control","max-age="+maxAge);
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());
        }
    }
//csc_012804_1_end

    /**
     * Write a DOM to a ServletResponse object. This method will
     * automatically set the content type for you.
     * This method is copied from DefaultDOMWriter. It should really be
     * abstracted from a common base class.
     *
     * @param node a DOM node, which belongs to the Document to be written out
     * @param resp the HttpServletResponse object
     */
    public void write(Node node, HttpServletResponse resp) throws IOException {
//csc_012804_1_start
/*
        //set the response type (TODO:need to support WML as well)
        String responseType = "text/xml";
        if (node instanceof HTMLDocument) responseType = "text/html";
        if (logger.isDebugEnabled()) logger.debug("Setting the response type:"+responseType);
        resp.setContentType(responseType);

        //if we need to prevent caching
        if (preventCaching) {
            //add the appropriate headers to the response
            if (logger.isDebugEnabled()) logger.debug("updating resp hdr to prevent caching");
            resp.setHeader("Pragma","no-cache");
            resp.setHeader("Cache-Control","no-cache");
            resp.setDateHeader("Expires", System.currentTimeMillis());

        //csc_061202.1 - added
        //otherwise explicitly give it a max-age (this will generally allow browsers like
        //IE to page back in history without reloading, but if the user actually revisits the
        //URL, then it will still be reloaded)
        } else {
            resp.setHeader("Cache-Control","max-age="+maxAge);
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());
        }
*/
//saw_031604_1        if (getContentType()==null) setContentType((node instanceof HTMLDocument) ? "text/html" : "text/xml");
        prepareResponse(node, resp);
//csc_012804_1_end

        //write the dom
        
      //saw_031604_1 begin
        // We want to use resp.getOutputStream() instead of resp.getWriter() because the PrintWriter
        // returned by getWriter() apparently has no buffer whereas the ServletOutputStream does.
        // This means that using the PrintWriter would cause the message chunks in the response to
        // be extremely small, with each call to print() causing a new chunk to be sent.
        
//        write(node, resp.getWriter());
        write(node, new OutputStreamWriter(resp.getOutputStream()));
      //saw_031604_1 end
    }

	/* (non-Javadoc)
     * @see org.barracudamvc.core.util.dom.DOMWriter#write(org.w3c.dom.Node, java.io.OutputStream)
     */
    public void write(Node node, OutputStream out) throws IOException {
        if (node.getNodeType()==Node.DOCUMENT_NODE) {
            DocumentFactory.getInstance().docManFor(node).serialize(out);
        } else {
          //saw_031604_1 begin
            // use OutputStreamWriter instead of PrintWriter... there is not need to introduce
            // buffering here. In fact, using PrintWriter as it was used here could cause some
            // output not to get printed because the gc may reclaim the PrintWriter before output is
            // flushed.

//            write(node, new PrintWriter(out));
            write(node, new OutputStreamWriter(out));
          //saw_031604_1 end
        }
    }

    /**
     * output of the Document of this node using a writer for the output.
     * After serialization, the Document is being repaired.
     */
    public void write(Node node, Writer writer) throws IOException {
        switch(node.getNodeType()) {
            case Node.DOCUMENT_NODE:
                DocumentFactory.getInstance().docManFor(node).serialize(writer);
                break;
            case Node.ELEMENT_NODE: 
                DocumentManager man = DocumentFactory.getInstance().docManFor(node);
                writer.write(man.serialize((Element)node, true));
                break;  //csc_012804_1 - shouldn't there be a break here???
            default:
                throw new IllegalArgumentException("Only Elements and Dcument can be serialized, not "+node);
        }
        if (!leaveWriterOpen) writer.close();   //csc_012804_1
    }

    /**
     * @return Returns the maxAge.
     */
    public int getMaxAge() {
        return maxAge;
    }

    /**
     * @param maxAge The maxAge to set.
     */
    public void setMaxAge(int maxAge) {
        this.maxAge = maxAge;
    }

    /**
     * @return Returns the preventCaching.
     */
    public boolean isPreventCaching() {
        return preventCaching;
    }

    /**
     * @param preventCaching The preventCaching to set.
     */
    public void setPreventCaching(boolean preventCaching) {
        this.preventCaching = preventCaching;
    }

}