/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LocalUtilities.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.examples.ex1;

import java.io.IOException;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.ClientSideRedirectException;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.DispatchQueue;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.ListenerFactory;
import org.barracudamvc.examples.ex1.events.GetLoginScreen;
import org.barracudamvc.examples.ex1.events.GetMainScreen;
import org.barracudamvc.examples.ex1.events.LocalRequestEvent;
import org.barracudamvc.examples.ex1.events.LoginScreenEvent;
import org.barracudamvc.plankton.data.ObjectRepository;


/**
 * General utility event handlers which apply to all screens
 * in the app. This really provides "master" controller functionality
 * which gets applied to all events getting dispatched into the system 
 * through the Http gateway.
 */
public class LocalUtilities extends DefaultEventGateway {

    //public constants
    protected static final Logger logger = Logger.getLogger(LocalUtilities.class.getName());

    //this defines the various event handlers
    private ListenerFactory localRequestFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new LocalRequestHandler();}        public String getListenerID() {return getID(LocalRequestHandler.class);}};                

    /**
     * Public constructor
     */
    public LocalUtilities() {
        //specify who's interested in what
        specifyLocalEventInterests(localRequestFactory, LocalRequestEvent.class);
    }
    
    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * HttpRequest - Here we redirect to the LoginScreen if necessary.
     * Make sure they aren't trying to dispatch a bogus event, and make sure
     * they have user/pwd info in their session.
     *
     * NOTE: if we were aiming for reusability, we could actually implement
     * this as multiple listeners in standalone classes, allowing a developer 
     * to pick and choose as needed...for now, we just kept it simple and lumped 
     * it all into one handler.
     */
    class LocalRequestHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            HttpServletRequest req = context.getRequest();
            DispatchQueue queue = context.getQueue();

            //figure out which event triggered this request
            logger.debug("Locating the event which triggered this one");
            List procList = queue.listProcessedEvents();
            BaseEvent triggerEvent = (BaseEvent) procList.get(procList.size()-1);
            logger.debug("Trigger event:"+triggerEvent);

            //see if we can validate user/pwd info
            HttpSession session = req.getSession();
            int valid = LoginServices.validateUserPwd(session);
            logger.debug("Checking for valid user/pwd info in the session...result:"+LoginServices.xrefValidity(valid));

            //if the event which triggered this is NOT an instance of 
            //LoginScreen event (meaning they're trying to do something 
            //OTHER than log in) and they don't have valid credentials,
            //redirect them to the LoginScreen
            if (valid!=LoginServices.VALID && (triggerEvent==null || (!(triggerEvent instanceof LoginScreenEvent)))) {
                logger.debug("Redirecting to the LoginScreen because they lack valid credentials");
                throw new ClientSideRedirectException(new GetLoginScreen());
            }
            
            //if the event which triggered this IS a LoginScreen event
            //(meaning they're trying to log in again) and they already
            //have valid login information, just redirect them onto the 
            //main screen
            if (triggerEvent!=null && (triggerEvent instanceof LoginScreenEvent) && valid==LoginServices.VALID) {
                logger.debug("Redirecting to MainScreen since they already have valid credentials");
                
//csc_082302.2_start
                //rather than store the value in the session, we can put it in the 
                //LocalRepository since that structure is persisted across the multiple
                //req-resp cycles that happen when we throw a ClientSideRedirectException.
                //If for some reason the lr and state information stop getting persisted 
                //(ie. if we inadvertantly introdcue a bug), then this auto-login
                //stuff will stop working
//                session.setAttribute(LoginServices.AUTO_LOGIN, ""+true);
                ObjectRepository lr = ObjectRepository.getLocalRepository();
                lr.putState(LoginServices.AUTO_LOGIN, Boolean.TRUE);
//csc_082302.2_end
                throw new ClientSideRedirectException(new GetMainScreen());
            }
        }
    }
}

