/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: HttpSessionStateMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.plankton.data;

import java.util.*;

import javax.servlet.http.HttpSession;

/**
 * The implementation provides a StateMap bridge to a HttpSession
 * object. By this, we mean that this class allows you to treat an
 * HttpSession as a StateMap. Unlike the HttpSession, this class 
 * will handle null keys, values.
 */
public class HttpSessionStateMap implements StateMap {

    //private vars
    protected HttpSession session = null;
    private String NULL = "~Null~";                    
    
    /**
     * Public constructor.
     *
     * @param isession the underlying servlet session structure
     */    
    public HttpSessionStateMap(HttpSession isession) {
        session = isession;
    }
    
    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        if (key==null) key = NULL;
        if (val==null) val = NULL;
        session.setAttribute(key.toString(),val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (key==null) key = NULL;
        Object val = session.getAttribute(key.toString());
        return ((val==null || val.equals(NULL)) ? null : val);
    }
    
    /**
     * remove a property in this StateMap. This function was expanded in csc_082103_1 to
     * support the notion of wildcarding, allowing you to remove multiple keys in one fell
     * swoop. Basically, if the key is a String, which ends with an '*', then any keys that
     * start with that string will be removed (and in this case, the method returns a Map of
     * key/val pairs that got removed, rather than a single object that got removed). This 
     * approach is not quite as flexible as using regular expressions, but that would make us 
     * dependent on jdk1.4 (so we won't go there for now). Note that this class backs the 
     * ObjectRepository data structures, so this functionality applies there as well.
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        try {
            //remove all keys that match
            if (key!=null && (key instanceof String) && ((String) key).endsWith("*")) {
                Map removed = new HashMap();
                String keystr = (String) key;
                String targetstr = keystr.substring(0,keystr.length()-1);
//csc_101304_1_start
/*
                List keys = getStateKeys();
                Iterator it = keys.iterator();
                while (it.hasNext()) {
                    Object okey = it.next();
                    if ((okey instanceof String) && ((String) okey).startsWith(targetstr)) {
                        Object oval = session.getAttribute(okey.toString());
                        removed.put(okey, oval);
                        session.removeAttribute(okey.toString());
                    }
                }
*/
                Iterator it = getStateKeys().iterator();
                while (it.hasNext()) {
                    String tkey = (String) it.next();
                    if (tkey!=null && tkey.startsWith(targetstr)) {
                        removed.put(tkey, session.getAttribute(tkey));
                        session.removeAttribute(tkey);
                    }
                }
//csc_101304_1_end            
                return (removed.size()>0 ? removed : null);        
            } else {
                if (key==null) key = NULL;
                Object val = session.getAttribute(key.toString());
                session.removeAttribute(key.toString());
                return ((val==null || val.equals(NULL)) ? null : val);
            }
        
        //this occasionally happens if the session expires as you are in the process of 
        //taking something out of it...in such a case, just catch the error and return null
        } catch (NullPointerException e) {
            return null;        
        
        //if the session has been invalidated, its possible an IllegalStateException will
        //be thrown. Just consume the exception and return null.    
        } catch (IllegalStateException e) {
            return null;        
        }
    }
    
    /**
     * get a keyset for this StateMap (whether or 
     * not the set is backed by the data store depends on 
     * the implementation). In this particular case, the set
     * is just a COPY of the underlying data structure
     *
     * @return a Set of keys for this StateMap
     */
    public Set getStateKeys() {
        Enumeration e = session.getAttributeNames();
//csc_101304_1_start
/*
        List list = new ArrayList();
        while (e.hasMoreElements())    {
            list.add(e.nextElement());
        }        
        return list;
*/
        Set set = new TreeSet();
        while (e.hasMoreElements()) {
            set.add(e.nextElement());
        }        
        return set;
//csc_101304_1_end
    }
    
    /**
     * get a Map that holds the state values (whether or 
     * not the Map is backed by the data store depends on 
     * the implementation). In this particular case, the map
     * is just a COPY of the underlying data store (session)
     *
     * @return a Map of key/val pairs for this StateMap
     */
    public Map getStateStore() {
//csc_101304_1_start
/*
        Map map = new HashMap();
//        Map map = new TreeMap();
        List keys = getStateKeys();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = session.getAttribute(key.toString());
            map.put(key,val);            
        }
        return map;        
*/        
        Enumeration e = session.getAttributeNames();
        Map map = new TreeMap();
        while (e.hasMoreElements()) {
            String key = (String) e.nextElement();
            map.put(key, getState(key));
        }        
        return map;
//csc_101304_1_end
    }

    /**
     * clear all state information
     */
    public void clearState() {
//csc_101304_1        List keys = getStateKeys();
        Set keys = getStateKeys();  //csc_101304_1
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            removeState(key);
        }
    }

    /**
     * get a reference to the underlying HttpSession
     *
     * @return a reference to the underlying HttpSession
     */    
    public HttpSession getSession() {
        return session;
    }
}
