/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ServletRequestParameterStateMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.plankton.data;

import java.util.*;

import javax.servlet.ServletRequest;

/**
 * The implementation provides a StateMap bridge to a ServletRequest
 * object's paramter values. By this, we mean that this class allows 
 * you to access ServletRequest paramters through the StateMap interface. 
 * Unlike the ServletRequest parameters, this class will handle null keys, 
 * values. Note however, that it's read-only. Put requests are silently ignored.
 */
public class ServletRequestParameterStateMap implements StateMap {

    //private vars
    protected ServletRequest request = null;
    private String NULL = "~Null~";                    
    
    /**
     * Public constructor.
     *
     * @param irequest the underlying servlet request structure
     */    
    public ServletRequestParameterStateMap(ServletRequest irequest) {
        request = irequest;
    }
    
    /**
     * set a property in this StateMap - not implemented because 
     * the servlet API doesn't allow you to set parameters
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        throw new UnsupportedOperationException();  //csc_052803_2
    }
    
    /**
     * get a property in this StateMap. Since the underlying structure is
     * an HttpRequest parameter we have to handle the case where there are 
     * multiple values for a key. In this case, a List will be returned.
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (key==null) key = NULL;
        Object[] vals = request.getParameterValues(key.toString());
        if (vals==null) return null;
        else if (vals.length==1) return (vals[0].equals(NULL) ? null : vals[0]);
        else {
            List list = new ArrayList();
            for (int i=0, max=vals.length; i<max; i++) {
                list.add(vals[i]);
            }
            return list;
        }
    }
    
    /**
     * remove a property in this StateMap - not implemented because 
     * the servlet API doesn't allow you to set parameters
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
//csc_052803_2        return null;
        throw new UnsupportedOperationException();  //csc_052803_2
    }
    
    /**
     * get a keyset for this StateMap (whether or 
     * not the set is backed by the data store depends on 
     * the implementation). In this particular case, the set
     * is just a COPY of the underlying data structure
     *
     * @return a Set of keys for this StateMap
     */
    public Set getStateKeys() {
        Enumeration e = request.getParameterNames();
//csc_101304_1_start
/*
        List list = new ArrayList();
        while (e.hasMoreElements())    {
            list.add(e.nextElement());
        }        
        return list;
*/
        Set set = new TreeSet();
        while (e.hasMoreElements())    {
            set.add(e.nextElement());
        }        
        return set;
//csc_101304_1_end
    }
    
    /**
     * get a Map that holds the state values (whether or 
     * not the Map is backed by the data store depends on 
     * the implementation). In this particular case, the map
     * is a COPY of the underlying data store itself (ServletRequest Parameters)
     *
     * @return a Map of key/val pairs for this StateMap
     */
    public Map getStateStore() {
//csc_101304_1_start
/*
        Map map = new HashMap();
//        Map map = new TreeMap();
        List keys = getStateKeys();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = getState(key);
            map.put(key,val);            
        }
        return map;        
*/        
        Enumeration e = request.getParameterNames();
        Map map = new TreeMap();
        while (e.hasMoreElements()) {
            String key = (String) e.nextElement();
            map.put(key, getState(key));
        }        
        return map;
//csc_101304_1_end
    }

    /**
     * clear all state information
     */
    public void clearState() {
        throw new UnsupportedOperationException();
    }

    /**
     * get a reference to the underlying ServletRequest
     *
     * @return a reference to the underlying ServletRequest
     */    
    public ServletRequest getRequest() {
        return request;
    }
}
