/*
 * ---------------------------------------------------------------------
 * This class is taken verbatim from a free implementation which is described
 * and documented here: http://archive.devx.com/java/free/articles/Kabutz01/Kabutz01-1.asp.
 * You may also wish to visit the author's website here: http://www.javaspecialists.co.za
 *
 * The only modification made has been to change the package for easy inclusion
 * in Barracuda's Plankton library, and to include the EPL licensing verbiage
 * ---------------------------------------------------------------------
 *
 * Copyright (C) 2003  Dr. Heinz M. Kabutz [h.kabutz@computer.org]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SoftHashMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.plankton.data;

//: SoftHashMap.java
import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;
import java.util.AbstractMap;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

public class SoftHashMap extends AbstractMap {

    /** The internal HashMap that will hold the SoftReference. */
    private final Map hash = new HashMap();
    /** The number of "hard" references to hold internally. */
    private final int HARD_SIZE;
    /** The FIFO list of hard references, order of last access. */
    private final LinkedList hardCache = new LinkedList();
    /** Reference queue for cleared SoftReference objects. */
    private final ReferenceQueue queue = new ReferenceQueue();

    public SoftHashMap() { this(100); }
    public SoftHashMap(int hardSize) { HARD_SIZE = hardSize; }

    public Object get(Object key) {
        Object result = null;
        // We get the SoftReference represented by that key
        SoftReference soft_ref = (SoftReference)hash.get(key);
        if (soft_ref != null) {
            // From the SoftReference we get the value, which can be
            // null if it was not in the map, or it was removed in
            // the processQueue() method defined below
            result = soft_ref.get();
            if (result == null) {
                // If the value has been garbage collected, remove the
                // entry from the HashMap.
                hash.remove(key);
            } else {
                // We now add this object to the beginning of the hard
                // reference queue.  One reference can occur more than
                // once, because lookups of the FIFO queue are slow, so
                // we don't want to search through it each time to remove
                // duplicates.
                hardCache.addFirst(result);
                if (hardCache.size() > HARD_SIZE) {
                    // Remove the last entry if list longer than HARD_SIZE
                    hardCache.removeLast();
                }
            }
        }
        return result;
    }

    /** We define our own subclass of SoftReference which contains
     not only the value but also the key to make it easier to find
     the entry in the HashMap after it's been garbage collected. */
    private static class SoftValue extends SoftReference {
        private final Object key; // always make data member final
        /** Did you know that an outer class can access private data
        members and methods of an inner class?  I didn't know that!
        I thought it was only the inner class who could access the
        outer class's private information.  An outer class can also
        access private members of an inner class inside its inner
        class. */
        private SoftValue(Object k, Object key, ReferenceQueue q) {
            super(k, q);
            this.key = key;
        }
    }

    /** Here we go through the ReferenceQueue and remove garbage
     collected SoftValue objects from the HashMap by looking them
     up using the SoftValue.key data member. */
    private void processQueue() {
        SoftValue sv;
        while ((sv = (SoftValue)queue.poll()) != null) {
            hash.remove(sv.key); // we can access private data!
        }
    }
    /** Here we put the key, value pair into the HashMap using
     a SoftValue object. */
    public Object put(Object key, Object value) {
        processQueue(); // throw out garbage collected values first
        return hash.put(key, new SoftValue(value, key, queue));
    }
    public Object remove(Object key) {
        processQueue(); // throw out garbage collected values first
        return hash.remove(key);
    }
    public void clear() {
        hardCache.clear();
        processQueue(); // throw out garbage collected values
        hash.clear();
    }
    public int size() {
        processQueue(); // throw out garbage collected values first
        return hash.size();
    }
    public Set entrySet() {
        processQueue(); // throw out garbage collected values first
        return hash.entrySet();
    }
}