/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestDateRangeValidator.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms.validators;

import java.util.*;
import java.text.*;

import junit.framework.*;
import org.apache.log4j.*;

import org.barracudamvc.core.forms.*;
import org.barracudamvc.plankton.data.*;
import org.barracudamvc.plankton.DateUtil;
import org.barracudamvc.testbed.*;

/**
 * This test verifies that DateRangeValidator works correctly.
 */
public class TestDateRangeValidator extends ValidatorTestCase {

	//common vars (customize for every test class)
	private static String testClass = TestDateRangeValidator.class.getName();
	private static Logger logger = Logger.getLogger("test."+testClass);

	//variables
	
	//-------------------- Basics --------------------------------

	/**
	 * Public Constructor
	 */
	public TestDateRangeValidator(String name) {
		super(name);
	}
	
	/**
	 * Every test class should have a main method so it can be run
	 * directly (when debugging tests you often will not want to run
	 * the whole suite)
	 *
	 * @param args defined in test.util.TestUtil
	 */
	public static void main(String args[]) {
		//check for standard runtime parameters
		TestUtil.parseParams(args);

		//launch the test
		if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
		else junit.swingui.TestRunner.main(new String[] {testClass});
	}

	public static Test suite() {

		TestSuite allLocalesSuite = new TestSuite();

		Locale[] availableLocales = Locale.getAvailableLocales();
		for (int i = 0; i < availableLocales.length; i++) {
            //csc_082603_2
            //ok, skip the thai locale until we have someone who knows enough about it to be able
            //to say what are valid dates and what aren't. Thai uses a solar calendar, which means
            //what we consider to be leap years aren't. This was causing test cases to fail, and since 
            //I don't know enough about thai to know what's actually correct, for now I am just removing
            //this from the test case.
            if (availableLocales[i].getLanguage().equals("th") && availableLocales[i].getCountry().equals("TH")) continue;
			
            //build the test suite 
			TestSuite suite = new TestSuite();
			suite.addTestSuite( TestDateRangeValidator.class );
			allLocalesSuite.addTest( new LocaleTestSuiteSetup( suite, availableLocales[i]) );
			logger.info("Adding:" + availableLocales[i].getCountry() );
		}

		return allLocalesSuite;
	}
	
	//-------------------- Actual Tests --------------------------
	//Note: all the methods herein should follow the testXXXX naming convention
	//Also keep in mind that local vars set in one test method are NOT retained
	//when the next method is invoked because JUnit makes a separate instance of
	//the test class for each testXXXX method!!!
	
	/**
	 * Test String types
	 */
	public void testString() {
		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);
		Date now = new Date();
		DateRangeValidator afterNowValidator = new DateRangeValidator(now,null);

        // ilc_022202.1_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("StringAfterNowInvalid1", "foo");
        sm.putState("StringAfterNowInvalid2", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
        sm.putState("StringAfterNowInvalid3", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
        sm.putState("StringAfterNowInvalid4", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
        // now should pass a range of today..null
        //new String( aDateFormat.format( now ) ),
        sm.putState("s2", new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime())));

        this.assertAllInvalid(afterNowValidator, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("StringAfterNowValid1", null);
        sm.putState("StringAfterNowValid2", "");
        sm.putState("StringAfterNowValid3", TestUtil.dateStringInDefaultLocaleShortForm("2033", "1", "1"));
        sm.putState("StringAfterNowValid4", TestUtil.dateStringInDefaultLocaleShortForm("2036", "2", "29"));
        sm.putState("StringAfterNowValid5", TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"));
        sm.putState("StringAfterNowValid6", new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime())));

        this.assertAllValid(afterNowValidator, sm, FormType.STRING);



        DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

        sm = null;
        sm = new DefaultStateMap();
        //isNull test strips characters
        //sm.putState("StringBeforeNowInValid1", "   ");
        sm.putState("StringBeforeNowInValid1", "foo");
        sm.putState("StringBeforeNowInValid2", TestUtil.dateStringInDefaultLocaleShortForm("2033", "1", "1"));
        sm.putState("StringBeforeNowInValid3", TestUtil.dateStringInDefaultLocaleShortForm("2036", "2", "29"));
        sm.putState("StringBeforeNowInValid4", TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"));
        sm.putState("StringBeforeNowInValid5", new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime())));

        this.assertAllInvalid(beforeNowValidator, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("StringBeforeNowValid1", null);
        sm.putState("StringBeforeNowValid2", "");
        sm.putState("StringBeforeNowValid3", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
        sm.putState("StringBeforeNowValid4", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
        sm.putState("StringBeforeNowValid5", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
        sm.putState("StringBeforeNowValid6", new String( aDateFormat.format( now )));
        sm.putState("StringBeforeNowValid7", new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime())));

        this.assertAllValid(beforeNowValidator, sm, FormType.STRING);



        DateRangeValidator todayValidator = new DateRangeValidator( DateUtil.getFirstDateToday().getTime(),
                                                                    DateUtil.getLastDateToday().getTime() );
        sm = null;
        sm = new DefaultStateMap();
        //isNull test strips characters
        //sm.putState("StringTodayInvalid1", "   ");
        sm.putState("StringTodayInvalid1", "foo");
        sm.putState("StringTodayInvalid2", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
        sm.putState("StringTodayInvalid3", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
        sm.putState("StringTodayInvalid4", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
        sm.putState("StringTodayInvalid5", TestUtil.dateStringInDefaultLocaleShortForm("2036", "2", "29"));
        sm.putState("StringTodayInvalid6", new String( aDateFormat.format( DateUtil.getLastDateYesterday().getTime())));
        sm.putState("StringTodayInvalid7", new String( aDateFormat.format( DateUtil.getFirstDateTomorrow().getTime())));

        this.assertAllInvalid(todayValidator, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("StringTodayValid1", null);
        sm.putState("StringTodayValid2", "");
        sm.putState("StringTodayValid3", new String( aDateFormat.format( new Date())));
        sm.putState("StringTodayValid4", new String( aDateFormat.format( DateUtil.getFirstDateToday().getTime())));
        sm.putState("StringTodayValid5", new String( aDateFormat.format( DateUtil.getLastDateToday().getTime())));

        this.assertAllValid(todayValidator, sm, FormType.STRING);
        // ilc_022202.1_end
	}

	public void testLong() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);

		// ilc_022202.3_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("LongInvalid1", new Long(0));
        sm.putState("LongInvalid2", new Long(1));
        sm.putState("LongInvalid3", new Long(-1));
        sm.putState("LongInvalid4", new Long(Long.MIN_VALUE));
        sm.putState("LongInvalid5", new Long(Long.MAX_VALUE));

        this.assertAllInvalid(v, sm, FormType.LONG);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("LongValid1", null);
        this.assertAllValid(v, sm, FormType.LONG);
        // ilc_022202.3_end
	}
	
	public void testBoolean() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);

		// ilc_022202.5_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("BooleanInvalid1", Boolean.FALSE);
        sm.putState("BooleanInvalid2", Boolean.TRUE);

        this.assertAllInvalid(v, sm, FormType.BOOLEAN);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("BooleanValid1", null);
        this.assertAllValid(v, sm, FormType.BOOLEAN);
        // ilc_022202.5_end
	}

	public void testShort() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);

		// ilc_022202.7_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("ShortInvalid1", new Short((short)0));
        sm.putState("ShortInvalid2", new Short((short)1));
        sm.putState("ShortInvalid3", new Short((short)-1));
        sm.putState("ShortInvalid4", new Short(Short.MIN_VALUE));
        sm.putState("ShortInvalid5", new Short(Short.MAX_VALUE));


        this.assertAllInvalid(v, sm, FormType.SHORT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("ShortValid1", null);
        this.assertAllValid(v, sm, FormType.SHORT);
        // ilc_022202.7_end
	}
	
	public void testInteger() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);

		// ilc_022202.9_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("IntegerInvalid1", new Integer(0));
        sm.putState("IntegerInvalid2", new Integer(1));
        sm.putState("IntegerInvalid3", new Integer(-1));
        sm.putState("IntegerInvalid4", new Integer(Integer.MIN_VALUE));
        sm.putState("IntegerInvalid5", new Integer(Integer.MAX_VALUE));

        this.assertAllInvalid(v, sm, FormType.INTEGER);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("IntegerValid1", null);
        this.assertAllValid(v, sm, FormType.INTEGER);
        // ilc_022202.9_end
	}
	
	public void testDate()  {
		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);
		Date now = new Date();
		DateRangeValidator afterNowValidator = new DateRangeValidator(now,null);

        // ilc_022202.11_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        try {
          sm.putState("DateAfterNowInvalid1", new Date(now.getTime()-1));
          sm.putState("DateAfterNowInvalid2", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
          sm.putState("DateAfterNowInvalid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
          sm.putState("DateAfterNowInvalid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));

          this.assertAllInvalid(afterNowValidator, sm, FormType.DATE);

          sm = null;
          sm = new DefaultStateMap();
          sm.putState("DateAfterNowValid1", null);
          sm.putState("DateAfterNowValid2", new Date());
          sm.putState("DateAfterNowValid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2033", "1", "1")));
          sm.putState("DateAfterNowValid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2036", "2", "29")));
          sm.putState("DateAfterNowValid5", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")));

          this.assertAllValid(afterNowValidator, sm, FormType.DATE);



          DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

          sm = null;
          sm = new DefaultStateMap();
          sm.putState("DateBeforeNowInValid1", new Date(now.getTime()+1));
          sm.putState("DateBeforeNowInValid2", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2033", "1", "1")));
          sm.putState("DateBeforeNowInValid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2036", "2", "29")));
          sm.putState("DateBeforeNowInValid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")));

          this.assertAllInvalid(beforeNowValidator, sm, FormType.DATE);

          sm = null;
          sm = new DefaultStateMap();
          sm.putState("DateBeforeNowValid1", null);
          sm.putState("DateBeforeNowValid2", new Date(now.getTime()-1));
          sm.putState("DateBeforeNowValid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
          sm.putState("DateBeforeNowValid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
          sm.putState("DateBeforeNowValid5", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));

          this.assertAllValid(beforeNowValidator, sm, FormType.DATE);



          DateRangeValidator todayValidator = new DateRangeValidator( DateUtil.getFirstDateToday().getTime(),
                                                                      DateUtil.getLastDateToday().getTime() );
          sm = null;
          sm = new DefaultStateMap();
          sm.putState("DateTodayInvalid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
          sm.putState("DateTodayInvalid2", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
          sm.putState("DateTodayInvalid3", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));;
          sm.putState("DateTodayInvalid4", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2036", "2", "29")));
          sm.putState("DateTodayInvalid5", DateUtil.getLastDateYesterday().getTime());
          sm.putState("DateTodayInvalid6", DateUtil.getFirstDateTomorrow().getTime());

          this.assertAllInvalid(todayValidator, sm, FormType.DATE);

          sm = null;
          sm = new DefaultStateMap();
          sm.putState("DateTodayValid1", null);
          sm.putState("DateTodayValid2", new Date());
          sm.putState("DateTodayValid3", DateUtil.getFirstDateToday().getTime());
          sm.putState("DateTodayValid4", DateUtil.getLastDateToday().getTime());

          this.assertAllValid(todayValidator, sm, FormType.DATE);

        } catch(java.text.ParseException ex) {
          fail( "testDate failed for locale:"+Locale.getDefault()+" because " + ex.getMessage() );
        }
        // ilc_022202.11_end
	}
	
	public void testDouble() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);

		// ilc_022202.13_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("DoubleInvalid1", new Double(0));
        sm.putState("DoubleInvalid2", new Double(1));
        sm.putState("DoubleInvalid3", new Double(-1));
        sm.putState("DoubleInvalid4", new Double(Double.MIN_VALUE));
        sm.putState("DoubleInvalid5", new Double(Double.MAX_VALUE));

        this.assertAllInvalid(v, sm, FormType.DOUBLE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("DoubleValid1", null);
        this.assertAllValid(v, sm, FormType.DOUBLE);
        // ilc_022202.13_end
	}
	
	public void testFloat() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);

		// ilc_022202.15_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("FloatInvalid1", new Float(0));
        sm.putState("FloatInvalid2", new Float(1));
        sm.putState("FloatInvalid3", new Float(-1));
        sm.putState("FloatInvalid4", new Float(Float.MIN_VALUE));
        sm.putState("FloatInvalid5", new Float(Float.MAX_VALUE));

        this.assertAllInvalid(v, sm, FormType.FLOAT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("FloatValid1", null);
        this.assertAllValid(v, sm, FormType.FLOAT);
        // ilc_022202.15_end
	}

}
