/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestNotNullValidator.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms.validators;

import java.util.*;

import org.barracudamvc.core.forms.*;
import org.barracudamvc.plankton.data.*;
import org.barracudamvc.testbed.*;


/**
 * This test verifies that NotNullValidator works correctly.
 */
public class TestNotNullValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestNotNullValidator.class.getName();

    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestNotNullValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();

        // ilc_022602.1_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        sm.putState("invalid not null2", "");
        sm.putState("invalid not null3", "   ");
        assertAllInvalid(v, sm, FormType.STRING);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null", "foofoo");
        assertAllValid(v, sm, FormType.STRING);

        // ilc_022602.1_end


        // ilc_022602.2_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertInvalid("Error invalidating ''==null", v, el, "");
        assertInvalid("Error invalidating '  '==null", v, el, "  ");
        assertValid("Error validating foo!=null", v, el, "foo");
        */
        // ilc_022602.2_end
    }
    
    /**
     * Test Boolean types - this test ensures that if we pass in a 
     * null FormElement or try to validate the length on a Boolean
     * it will always generate a ValidationException
     */
    public void testBoolean() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();

        // ilc_022602.3_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.BOOLEAN);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", new Boolean(true));
        sm.putState("valid not null2", new Boolean(false));
        assertAllValid(v, sm, FormType.BOOLEAN);

        // ilc_022602.3_end


        // ilc_022602.4_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating true!=null", v, el, new Boolean(true));
        assertValid("Error validating false!=null", v, el, new Boolean(false));
        */
        // ilc_022602.4_end
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        Integer i1 = new Integer(123);

        // ilc_022602.5_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.INTEGER);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", i1);
        assertAllValid(v, sm, FormType.INTEGER);

        // ilc_022602.5_end


        // ilc_022602.6_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating i1!=null", v, el, i1);
        */
        // ilc_022602.6_end
    }

    /**
     * Test Date types - this test ensures that if we try to validate 
     * the length on a Date it will always generate a ValidationException
     */
    public void testDate() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        Date d = new Date();

        // ilc_022602.7_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.DATE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", d);
        assertAllValid(v, sm, FormType.DATE);

        // ilc_022602.7_end


        // ilc_022602.8_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating d!=null", v, el, d);
        */
        // ilc_022602.8_end
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        Long l1 = new Long(123);

        // ilc_022602.9_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.LONG);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", l1);
        assertAllValid(v, sm, FormType.LONG);

        // ilc_022602.9_end


        // ilc_022602.10_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating l1!=null", v, el, l1);
        */
        // ilc_022602.10_end
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        Short s1 = new Short((short)123);

        // ilc_022602.11_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.SHORT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", s1);
        assertAllValid(v, sm, FormType.SHORT);

        // ilc_022602.11_end


        // ilc_022602.12_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating s1!=null", v, el, s1);
        */
        // ilc_022602.12_end
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        Double d1 = new Double(123);

        // ilc_022602.13_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.DOUBLE);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", d1);
        assertAllValid(v, sm, FormType.DOUBLE);

        // ilc_022602.13_end


        // ilc_022602.14_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating d1!=null", v, el, d1);
        */
        // ilc_022602.14_end
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        NotNullValidator v = new NotNullValidator();
        Float f1 = new Float(123);

        // ilc_022602.15_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();

        sm.putState("invalid not null1", null);
        assertAllInvalid(v, sm, FormType.FLOAT);

        sm = null;
        sm = new DefaultStateMap();
        sm.putState("valid not null1", f1);
        assertAllValid(v, sm, FormType.FLOAT);

        // ilc_022602.15_end


        // ilc_022602.16_start
        /* use state maps instead
        //v
        assertInvalid("Error invalidating null==null", v, el, null);
        assertValid("Error validating f1!=null", v, el, f1);
        */
        // ilc_022602.16_end
    }
    
}
